import logging

from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from react.models.fragments import Fragments
from react.models.pages import Pages

logger = logging.getLogger(__name__)


class Page_fragmentsQuerySet(AuditQuerySet):
    pass


class Page_fragmentsManager(AuditManager):

    @classmethod
    def getRecord(cls, record ) :
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Page_fragmentsQuerySet(self.model, using=self._db)


class Page_fragments(AuditModel):
    fragment = ForeignKeyProtect(Fragments)
    page = ForeignKeyProtect(Pages)

    objects = Page_fragmentsManager()

    def __str__(self):
        return f'ID:{self.id}, fragment: [{self.fragment}], page: [{self.page}]'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс таблица'
        unique_together = (('page', 'fragment'),)



# Generated by Django 3.2.6 on 2021-08-23 18:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0115_alter_images_size'),
    ]

    operations = [
        migrations.AddField(
            model_name='image_types',
            name='key',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



# Generated by Django 2.2 on 2019-04-14 07:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0009_auto_20190414_0756'),
    ]

    operations = [
        migrations.RenameField(
            model_name='messages',
            old_name='deleted',
            new_name='deleted_at',
        ),
        migrations.RenameField(
            model_name='messages_state',
            old_name='deleted',
            new_name='deleted_at',
        ),
        migrations.RenameField(
            model_name='messages_theme',
            old_name='deleted',
            new_name='deleted_at',
        ),
    ]



from django.urls import path

from one_c.views import entity_1c

urlpatterns = [

    path('Entity_1c/Fetch/', entity_1c.Entity_1c_Fetch),
    path('Entity_1c/Add', entity_1c.Entity_1c_Add),
    path('Entity_1c/Update', entity_1c.Entity_1c_Update),
    path('Entity_1c/Remove', entity_1c.Entity_1c_Remove),
    path('Entity_1c/Lookup/', entity_1c.Entity_1c_Lookup),
    path('Entity_1c/Info/', entity_1c.Entity_1c_Info),

]



# Generated by Django 2.2.5 on 2019-09-07 04:38

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0059_auto_20190905_0519'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='messages_files',
            name='message',
        ),
        migrations.CreateModel(
            name='Messages_files_refs',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('message', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='tracker.Messages')),
                ('messages_file', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='tracker.Messages_files')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
            },
        ),
    ]



from django.db.models import CharField
from django.utils.translation import gettext_lazy as _


class CaptionField(CharField):
    description = _("Наименованиме")

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('max_length', 255)
        kwargs.setdefault('db_index', True)
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        del kwargs["max_length"]
        del kwargs["db_index"]
        return name, path, args, kwargs



# Generated by Django 3.0.3 on 2020-02-06 06:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0078_delete_messages_theme_with_out_user_view'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='messages',
            options={'ordering': ('lastmodified',), 'verbose_name': 'Сообщения'},
        ),
        migrations.RemoveField(
            model_name='messages',
            name='date_create',
        ),
    ]



from django.urls import path

from isc_common.auth.views import user_e_mails

urlpatterns = [

    path('User_e_mails/Fetch/', user_e_mails.User_e_mails_Fetch),
    path('User_e_mails/Add', user_e_mails.User_e_mails_Add),
    path('User_e_mails/Update', user_e_mails.User_e_mails_Update),
    path('User_e_mails/Remove', user_e_mails.User_e_mails_Remove),
    path('User_e_mails/Lookup/', user_e_mails.User_e_mails_Lookup),
    path('User_e_mails/Info/', user_e_mails.User_e_mails_Info),
    path('User_e_mails/Copy', user_e_mails.User_e_mails_Copy),

]



import logging
import os

from code_generator.app.app import App
from code_generator.app.app_mailwindow import AppMainWindow
from code_generator.app.model import Model

logger = logging.getLogger(__name__)


class WinAppModel(Model):
    name = None
    mainWindow = None
    appImageDir = None
    isc_templ = 'Isc.pmd'
    startfunction_templ = 'start_function.pmd'

    def write_entity(self):
        for key, value in self.get_model_attributes():
            if isinstance(value, App):
                setattr(value, 'name_app', self.name)
                setattr(value, 'code_app', f'{self.name}.{key}')
                setattr(value, 'path_app', f'{self.name}{os.sep}{key}')
                value.write_entity()

    def __init__(self, *args: object, **kwargs: object):
        super().__init__(*args, **kwargs)

        if self.name is None:
            raise Exception(f'Не задано имя модели.')

        logger.debug(f'Инициализация приложения: {self.dbl_qutes_str(self.name)}')





# Generated by Django 3.2.4 on 2021-07-26 07:36

import django.utils.timezone
from django.db import migrations, models

import isc_common.fields.related


class Migration(migrations.Migration):
    dependencies = [
        ('react', '0003_fragment_params'),
    ]

    operations = [
        migrations.CreateModel(
            name='Fragment_param_types',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
            ],
            options={
                'verbose_name': 'Типы рараметров фрагмента',
            },
        ),
        migrations.AddField(
            model_name='fragment_params',
            name='type',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='react.fragment_param_types'),
            preserve_default=False,
        ),
    ]



import logging

from bitfield import BitField
from django.db.models import TextField, PositiveIntegerField

from isc_common.auth.models.user import User
from isc_common.fields.code_field import CodeStrictField
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditQuerySet, AuditManager, AuditModel
from isc_common.number import DelProps

logger = logging.getLogger(__name__)


class ProgressesQuerySet(AuditQuerySet):
    pass


class ProgressesManager(AuditManager):
    @classmethod
    def props(cls):
        return BitField(flags=(
            ('showCancel', 'Показать кнопку отменить процесс'),
        ), default=0, db_index=True)

    def resoreFromRequest(self, request, removed=None, ):
        from isc_common.progress import Progress

        request = DSRequest(request=request)
        for proces in [ProgressesManager.getRecord(record) for record in Progresses.objects.filter(user=request.user)]:
            Progress(
                qty=proces.get('qty'),
                user=request.user,
                id=proces.get('id_progress'),
                message=proces.get('message'),
                label_contents=proces.get('label_contents'),
                title=proces.get('title'),
                props=proces.get('props')
            )

    @classmethod
    def getRecord(cls, record):
        res = {
            'cnt': record.cnt,
            'deliting': record.deliting,
            'editing': record.editing,
            'id': record.id,
            'id_progress': record.id_progress,
            'label_contents': record.label_contents,
            'path': record.path,
            'props': record.props,
            'qty': record.qty,
            'title': record.title,
        }
        return DelProps(res)

    def get_queryset(self):
        return ProgressesQuerySet(self.model, using=self._db)


class Progresses(AuditModel):
    cnt = PositiveIntegerField()
    id_progress = CodeStrictField()
    label_contents = TextField()
    message = TextField()
    qty = PositiveIntegerField()
    props = ProgressesManager.props()
    title = TextField()
    path = CodeStrictField(default='progress')
    user = ForeignKeyCascade(User)

    objects = ProgressesManager()

    def __str__(self):
        return f'ID:{self.id}, title: {self.title}, qty: {self.qty}, cnt: {self.cnt}, id_progress: {self.id_progress}, user: [{self.user}]'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Отражение запущенных процессов'
        unique_together = (('user', 'id_progress'),)



from django.apps import apps
from django.core.files.storage import Storage


class StorageEx(Storage):
    def _get_model_cls(self, model_class_path):
        app_label, model_name = model_class_path.rsplit('.', 1)
        model = apps.get_model(app_label, model_name)
        return model



import logging

from django.conf import settings
from django.contrib.postgres.fields import ArrayField
from django.db import connection
from django.db.models import Model , BooleanField , BigAutoField , DateTimeField , Transform , CharField , TextField , BigIntegerField
from django.utils import timezone
from isc_common import setAttr , delAttr
from isc_common.fields.code_field import CodeField
from isc_common.managers.common_manager import CommonManager , CommonQuerySet
from isc_common.number import GetListFromStringCortege
from transliterate import translit
from transliterate.exceptions import LanguageDetectionError

logger = logging.getLogger( __name__ )


class Model_withOldId( Model ) :
    old_id = BigIntegerField( db_index=True , null=True , blank=True , unique=True )

    class Meta :
        abstract = True


class Model_withOldIdStr( Model ) :
    old_id = CodeField( unique=True )

    class Meta :
        abstract = True


class Model_withOldIds( Model ) :
    old_ids = ArrayField( BigIntegerField() , null=True , blank=True , unique=True )

    class Meta :
        abstract = True


class AuditQuerySet( CommonQuerySet ) :
    def getOptional( self , *args , **kwargs ) :
        try :
            return self.get( *args , **kwargs )
        except self.model.DoesNotExist :
            return None

    def _check_ed_izm_qty( self , **kwargs ) :
        if kwargs.get( 'ed_izm' ) is not None and kwargs.get( 'qty' ) is None or kwargs.get( 'ed_izm' ) is None and kwargs.get( 'qty' ) is not None :
            raise Exception( f'Единица измерения не может быть указана без количества' )

    def __init__( self , model=None , query=None , using=None , hints=None , alive_only=True ) :
        self.alive_only = alive_only
        super().__init__( model=model , query=query , using=using , hints=hints , alive_only=alive_only )

    def rearrange_parent( self , json ) :
        # _criteria = json.get('data').get('criteria')
        # if isinstance(_criteria, list):
        #     for criterion in _criteria:
        #         if criterion.get('fieldName') == 'parent_id':
        #             item_id = criterion.get('value')
        #             if (item_id, int):
        #                 criteria_lst = [criterion for criterion in _criteria if criterion.get('fieldName') == 'parent_id' and criterion.get('value') is not None]
        #                 if len(criteria_lst) > 0:
        #                     setAttr(json.get('data'), 'criteria', criteria_lst)
        return json

    def soft_delete( self ) :
        res = super().update( deleted_at=timezone.now() )
        if res > 0 :
            return self
        else :
            return None

    def soft_restore( self ) :
        res = super().update( deleted_at=None )
        return res

    def hard_delete( self ) :
        res = super().delete()
        return res

    def alive( self ) :
        return self.filter( deleted_at=None )

    def dead( self ) :
        return self.exclude( deleted_at=None )

    def update( self , **kwargs ) :
        if kwargs.get( 'lastmodified' ) is None :
            setAttr( kwargs , 'lastmodified' , timezone.now() )
        return super().update( **kwargs )


class AuditManager( CommonManager ) :
    @classmethod
    def getRecord( cls , record ) :
        return dict()

    def __init__( self , *args , **kwargs ) :
        self.alive_only = kwargs.pop( 'alive_only' , True )
        super().__init__( *args , **kwargs )

    def get_queryset( self ) :
        if self.alive_only :
            return AuditQuerySet( model=self.model , alive_only=self.alive_only ).filter( deleted_at=None )
        return AuditQuerySet( model=self.model , alive_only=self.alive_only )

    def hard_delete( self ) :
        return self.get_queryset().hard_delete()

    def soft_delete( self ) :
        return self.get_queryset().soft_delete()

    def soft_restore( self ) :
        return self.get_queryset().soft_restore()

    def getOptional( self , *args , **kwargs ) :
        try :
            return self.get( *args , **kwargs )
        except self.model.DoesNotExist :
            return None

    def getOr( self , *args , **kwargs ) :
        alternative = kwargs.get( 'alternative' )
        try :
            delAttr( kwargs , 'alternative' )
            return self.get( *args , **kwargs )
        except self.model.DoesNotExist :
            if not callable( alternative ) :
                raise Exception( 'alternative must be callable' )
            return alternative()
        except self.model.MultipleObjectsReturned as ex :
            old_ids = kwargs.get( 'old_ids__overlap' )
            old_id = kwargs.get( 'old_id' )
            if old_ids is None and old_id is None :
                if not callable( alternative ) :
                    raise Exception( 'alternative must be callable' )
                return alternative()

            if len( old_ids ) > 1 :
                raise ex

            old_id = old_ids[ 0 ]

            for item in self.filter( *args , **kwargs ) :
                if len( item.old_ids ) > 1 :
                    item.old_ids = list( filter( lambda x : x != old_id , item.old_ids ) )
                    item.save()

            return self.get( *args , **kwargs )

    def get_record( self , record ) :
        record = self.get( id=record.id if isinstance( record , Model ) else record.get( 'id' ) )
        self.getRecord( record=record )


class Manager( CommonManager ) :
    def __init__( self , *args , **kwargs ) :
        super().__init__( *args , **kwargs )

    def get_queryset( self ) :
        return AuditQuerySet( model=self.model )


class AuditModel( Model ) :
    id = BigAutoField( primary_key=True , verbose_name="Идентификатор" )
    deleted_at = DateTimeField( verbose_name="Дата мягкого удаления" , null=True , blank=True , db_index=True )
    editing = BooleanField( verbose_name="Возможность редактирования" , default=True )
    deliting = BooleanField( verbose_name="Возможность удаления" , default=True )
    lastmodified = DateTimeField( verbose_name='Последнее обновление' , editable=False , db_index=True , default=timezone.now )

    @classmethod
    def get_urls_datas( cls , record , keyimages , model , model_images , main_model , imports , django_model , django_model_images , code=None , add_params=[ ] ) :
        from isc_common.number import DelProps

        if not isinstance( record , dict ) :
            raise Exception( 'Record must be dict.' )

        record = DelProps( record )
        res = list()
        idx = 0
        for keyimage in keyimages :
            real_name , image_src = cls.get_urls_data(
                add_params=add_params[ idx ] if len( add_params ) != 0 else '' ,
                code=code ,
                django_model=django_model ,
                django_model_images=django_model_images ,
                id=record.get( 'id' ) ,
                imports=imports ,
                keyimage=keyimage ,
                main_model=main_model ,
                model=model ,
                model_images=model_images ,
            )
            idx += 1

            real_name , image_id = GetListFromStringCortege( real_name )
            image_src = f'{settings.IMAGE_CONTENT_PROTOCOL}://{settings.IMAGE_CONTENT_HOST}:{settings.IMAGE_CONTENT_PORT}/{image_src}'
            item = {
                f'{keyimage}_image_id' : image_id ,
                f'{keyimage}_image_src' : image_src ,
                f'{keyimage}_real_name' : real_name ,
            }
            res.append( item )

        for urls_data in res :
            for k , v in urls_data.items() :
                setAttr( record , k , v )
        return record

    @classmethod
    def get_urls_data( cls , id , keyimage , model , model_images , main_model , imports , django_model , django_model_images , code=None , add_params='' ) :
        if code is None :
            code = main_model

        if add_params :
            add_params = f'&{add_params}'

        _imports = ','.join( map( lambda x : x.replace( ' ' , '__' ) , imports ) )

        with connection.cursor() as cursor :
            sql_str = f'''select (select (ici.real_name, usi.image_id)
                                        from {model_images} as usi
                                                 join isc_common_images ici on ici.id = usi.image_id
                                                 join isc_common_image_types icit on usi.type_id = icit.id
                                        where usi.main_model_id = cd.id
                                          and icit.code = %s
                                          and icit.keyimage = %s
                                          and usi.deleted_at is null
                                        limit 1)                                                                                                           as real_name,
                                       concat('logic/Imgs/Download/', (
                                           select usi.image_id
                                           from {model_images} as usi
                                                    join isc_common_images ici on ici.id = usi.image_id
                                                    join isc_common_image_types icit on usi.type_id = icit.id
                                           where usi.main_model_id = cd.id
                                             and icit.code = %s
                                             and icit.keyimage = %s
                                             and usi.deleted_at is null
                                           limit 1), '?', 
                                           'code={code}', 
                                           '&', 
                                           'keyimage={keyimage}', 
                                           '&', 
                                           'path={main_model}', 
                                           '&', 
                                           'main_model={main_model}', 
                                           '&', 
                                           'main_model_id=', cd.id, 
                                           '&', 
                                           'imports=', '{_imports}', 
                                           '&', 
                                           'django_model=', '{django_model._meta.object_name}',
                                           '&', 
                                           'django_model_images=', '{django_model_images._meta.object_name}',
                                           '{add_params}'
                                           ) as image_src
                                from {model} as cd
                                where cd.id = %s'''

            cursor.execute( sql_str , [ code , keyimage , code , keyimage , id ] )
            record = cursor.fetchone()
            return record

    @classmethod
    def uncapitalize( cls , str ) :
        from isc_common.common.functions import uncapitalize
        return uncapitalize( str )

    @classmethod
    def translit( cls , value ) :
        if isinstance( value , int ) :
            return str( value )
        try :
            return cls.uncapitalize( translit( value , reversed=True ).replace( "'" , '' ).replace( ' ' , '_' ).replace( '.' , '_' ) )
        except LanguageDetectionError :
            return cls.uncapitalize( value.replace( "'" , '' ).replace( ' ' , '_' ).replace( '.' , '_' ) )

    @classmethod
    def dbl_qutes_str( cls , str ) :
        from isc_common.common.functions import dbl_qutes_str
        return dbl_qutes_str( str )

    @classmethod
    def qutes_str( cls , str ) :
        from isc_common.common.functions import qutes_str
        return qutes_str( str )

    @property
    def idHidden( self ) :
        return not self is None

    @property
    def is_deleted( self ) :
        return self.deleted_at is not None

    class Meta :
        abstract = True

    def soft_delete( self ) :
        self.deleted_at = timezone.now()
        self.save()
        return self

    def soft_restore( self ) :
        self.deleted_at = None
        self.save()
        return self

    objects = AuditManager()
    all_objects = AuditManager( alive_only=False )


class Dbl_spacesValue( Transform ) :
    lookup_name = 'delete_dbl_spaces'
    function = 'delete_dbl_spaces'


CharField.register_lookup( Dbl_spacesValue )
TextField.register_lookup( Dbl_spacesValue )


class Trim_Value( Transform ) :
    lookup_name = 'trim'
    function = 'trim'


CharField.register_lookup( Trim_Value )
TextField.register_lookup( Trim_Value )


class Upper_Value( Transform ) :
    lookup_name = 'upper'
    function = 'upper'


CharField.register_lookup( Upper_Value )
TextField.register_lookup( Upper_Value )



# Generated by Django 3.0.7 on 2020-06-12 22:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('reports', '0008_auto_20200612_1625'),
    ]

    operations = [
        migrations.RenameField(
            model_name='jasper_reports',
            old_name='url',
            new_name='path',
        ),
    ]



import logging

from django.db.models import Model, PositiveIntegerField
from transliterate import translit
from transliterate.exceptions import LanguageDetectionError

from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyProtect
from one_c.models.entity_1c import Entity_1c

logger = logging.getLogger(__name__)


class One_c_params_entity_view(Model):
    code = CodeField(verbose_name='Код параметтра')
    entity = ForeignKeyProtect(Entity_1c, verbose_name='Сущность 1С')

    @property
    def translited_code(self):
        try:
            return translit(self.code, reversed=True).replace("'", '').replace("'", '')
        except LanguageDetectionError:
            return self.code

    def __str__(self):
        return f"{self.id}"

    class Meta:
        managed = False
        db_table = 'one_c_params_entity_view'
        verbose_name = 'Все параметры 1С документов'



# Generated by Django 3.2 on 2021-04-09 04:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('history', '0014_auto_20200808_0428'),
    ]

    operations = [
        migrations.AlterField(
            model_name='history',
            name='id',
            field=models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор'),
        ),
    ]



# Generated by Django 2.1.5 on 2019-01-19 23:40

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Messages',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('checksum', isc_common.fields.code_field.CodeField(unique=True, verbose_name='Checksum MD5')),
                ('message', isc_common.fields.description_field.DescriptionField(verbose_name='Тело сообщения')),
            ],
            options={
                'verbose_name': 'Сообщения',
            },
        ),
        migrations.CreateModel(
            name='Messages_state',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField()),
                ('name', isc_common.fields.name_field.NameField()),
                ('description', isc_common.fields.description_field.DescriptionField()),
            ],
            options={
                'verbose_name': 'Состояние сообщений',
            },
        ),
        migrations.CreateModel(
            name='Messages_theme',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted', models.BooleanField(db_index=True, default=False, verbose_name='Признак удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField()),
                ('name', isc_common.fields.name_field.NameField()),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='tracker.Messages_theme')),
            ],
            options={
                'verbose_name': 'Темы трекинга',
            },
        ),
        migrations.AddField(
            model_name='messages',
            name='state',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='tracker.Messages_state'),
        ),
        migrations.AddField(
            model_name='messages',
            name='theme',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='tracker.Messages_theme'),
        ),
        migrations.AddField(
            model_name='messages',
            name='user',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 3.2 on 2021-04-11 16:21

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0095_users_images_old_id'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='users_images',
            name='old_id',
        ),
        migrations.AddField(
            model_name='users_images',
            name='old_ids',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.BigIntegerField(), blank=True, null=True, size=None, unique=True),
        ),
    ]



# Generated by Django 3.1.6 on 2021-02-20 05:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0070_auto_20210220_0519'),
    ]

    operations = [
        migrations.DeleteModel(
            name='AuditModelWithOldId',
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-13 10:55

import bitfield.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('twits', '0015_auto_20190909_1357'),
    ]

    operations = [
        migrations.AlterField(
            model_name='chat_user_user',
            name='common_id',
            field=models.PositiveIntegerField(default=None),
        ),
        migrations.AlterField(
            model_name='chat_users',
            name='props',
            field=bitfield.models.BitField((('compulsory_reading', 'Обязательное прочтение'), ('relevant', 'Актуальность')), db_index=True, default=0),
        ),
    ]



# Generated by Django 3.2.7 on 2021-09-07 19:04

from django.db import migrations
from isc_common.common.functions import check_image_files


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0129_alter_images_unique_together'),
    ]

    operations = [
        migrations.RunPython(check_image_files)
    ]



# Generated by Django 2.2.4 on 2019-08-21 01:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0008_auto_20190821_0109'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='day',
            options={'verbose_name': 'День'},
        ),
    ]



import logging

from crypto.models.upload_file import DSResponse_UploadFile
from isc_common.datetime import DateToStr
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.ws.webSocket import WebSocket
from tracker.models.messages import Messages
from twits import ChatData, ChatResponseData

logger = logging.getLogger(__name__)


class DSResponse__Messages_Confirm_reading(DSResponse_UploadFile):

    def __init__(self, request):
        json_data = request.POST.dict()
        # logger.debug(f'json_data: {json_data}')

        chat_data = ChatData(json_data)

        try:
            for message in Messages.objects.filter(guid=chat_data.guid,
                                           to_whom_id=chat_data.user_id,
                                           state__in=[
                                               chat_data.message_state_not_readed_id,
                                           ]):
                message.state_id = chat_data.message_state_readed_id
                date = DateToStr(message.lastmodified, '%d.%m.%Y, %H:%M:%S')
                _message = ChatResponseData(
                    channel=chat_data.channel,
                    date=date,
                    guid=str(message.guid).upper(),
                    message=message.message,
                    state__name=message.state.name,
                    to_whom_id=message.to_whom.id,
                    type=chat_data.type,
                    user__color=message.user.color,
                    user__short_name=message.user.get_short_name,
                    user_id=message.user.id,
                )
                WebSocket.send_message(host=chat_data.host, port=chat_data.port, channel=chat_data.channel, message=_message, logger=logger)
                message.save()
                #TODO : НЕ идет отсюда сообщение, т.е. неизвестно куда приходит , посмотреть по channel consummers
        except Exception as ex:
            raise ex

        self.response = dict(status=RPCResponseConstant.statusSuccess)



from django.urls import path

from react.views import fragments_item_types

urlpatterns = [

    path('Fragments_item_types/Fetch/', fragments_item_types.Fragments_item_types_Fetch),
    path('Fragments_item_types/Add', fragments_item_types.Fragments_item_types_Add),
    path('Fragments_item_types/Update', fragments_item_types.Fragments_item_types_Update),
    path('Fragments_item_types/Remove', fragments_item_types.Fragments_item_types_Remove),
    path('Fragments_item_types/Lookup/', fragments_item_types.Fragments_item_types_Lookup),
    path('Fragments_item_types/Info/', fragments_item_types.Fragments_item_types_Info),
    path('Fragments_item_types/Copy', fragments_item_types.Fragments_item_types_Copy),

]



# Generated by Django 2.1.7 on 2019-02-14 16:47

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0006_messages_parent'),
    ]

    operations = [
        migrations.AddField(
            model_name='messages',
            name='date_create',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, verbose_name='Дата записи'),
        ),
    ]



from django.urls import path

from isc_common.views import images
from isc_common.views.images import Images_Upload

urlpatterns = [

    path('Images/Fetch/', images.Images_Fetch),
    path('Images/Add', images.Images_Add),
    path('Images/Update', images.Images_Update),
    path('Images/Remove', images.Images_Remove),
    path('Images/Lookup/', images.Images_Lookup),
    path('Images/Info/', images.Images_Info),
    path('Images/Copy', images.Images_Copy),
    path('Images/Upload', Images_Upload),

]



import logging

from django.db.models import Model, PositiveIntegerField

from isc_common.fields.code_field import CodeField

logger = logging.getLogger(__name__)


class Mat_view(Model):
    schemaname = CodeField()
    relname = CodeField(primary_key=True)
    ownername = CodeField()
    refresh_order = PositiveIntegerField()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Mat Views'
        managed = False
        db_table = 'mat_view_refresh_order'



# Generated by Django 3.2.6 on 2021-08-23 11:26

from django.db import migrations


class Migration( migrations.Migration ) :
    dependencies = [
        ('isc_common' , '0108_alter_users_old_images_unique_together') ,
    ]

    operations = [
        migrations.RunSQL( '''DROP view all_tables;
                            CREATE OR REPLACE VIEW all_tables AS
                            SELECT table_name FROM information_schema.tables
                            WHERE table_schema NOT IN ('information_schema','pg_catalog')''' ) ,
    ]



from isc_common.auth.models.usergroup import UserGroup, GroupManager
from isc_common.common import developers
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse


@JsonResponseWithException(printing=False, printing_res=False)
def UserGroup_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=UserGroup.objects.
                # exclude(code=developers).
                exclude().
                get_range_rows1(
                request=request,
                function=GroupManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException(printing=False)
def UserGroup_Add(request):
    return JsonResponse(DSResponseAdd(data=UserGroup.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def UserGroup_Update(request):
    return JsonResponse(DSResponseUpdate(data=UserGroup.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def UserGroup_Remove(request):
    return JsonResponse(DSResponse(request=request, data=UserGroup.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def UserGroup_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=UserGroup.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def UserGroup_Info(request):
    return JsonResponse(DSResponse(request=request, data=UserGroup.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.2.7 on 2021-09-16 12:27

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('isc_common', '0133_auto_20210916_1226'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Widgets_trees',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('id_widget', isc_common.fields.code_field.CodeField(blank=True, null=True, unique=True)),
                ('name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('class_name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('structure', isc_common.fields.code_field.JSONFieldIVC(default=dict)),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='permission.widgets_trees')),
            ],
            options={
                'verbose_name': 'Таблица сохранения деревьев виджетов объектов системы',
            },
        ),
        migrations.CreateModel(
            name='Usergroup_permission',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('permission', isc_common.fields.code_field.JSONFieldIVC(default=dict)),
                ('usergroup', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='isc_common.usergroup')),
                ('widget', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='permission.widgets_trees')),
            ],
            options={
                'verbose_name': 'Доступы для групп',
                'unique_together': {('usergroup', 'widget')},
            },
        ),
        migrations.CreateModel(
            name='User_permission',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('permission', isc_common.fields.code_field.JSONFieldIVC(default=dict)),
                ('user', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
                ('widget', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='permission.widgets_trees')),
            ],
            options={
                'verbose_name': 'Доступы для пользователей',
                'unique_together': {('user', 'widget')},
            },
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-09 14:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0038_auto_20200209_1419'),
    ]

    operations = [
        migrations.AddField(
            model_name='progresses',
            name='guid',
            field=models.UUIDField(default=None),
        ),
    ]



# Generated by Django 3.2.6 on 2021-08-29 13:47

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0123_alter_users_images_main_model'),
    ]

    operations = [
        migrations.AlterField(
            model_name='users_images',
            name='main_model',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 3.1.6 on 2021-02-20 05:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0069_auditmodelwitholdid'),
    ]

    operations = [
        migrations.AlterField(
            model_name='auditmodelwitholdid',
            name='old_id',
            field=models.BigIntegerField(default=None),
        ),
    ]



import configparser
import logging
from tkinter import *

from django.core.management import BaseCommand
from django.db import connection

from one_c.models.entity_1c import Entity_1c
from one_c.models.mat_view import Mat_view
from one_c.models.one_c_params_entity_view import One_c_params_entity_view

logger = logging.getLogger(__name__)


class ExceptionOnDocLoading(Exception):
    pass


class Command(BaseCommand):
    def dbl_qutes_str(self, str):
        return f'"{str}"'

    def handle(self, *args, **options):
        try:
            for mat_view in Mat_view.objects.filter():
                with connection.cursor() as cursor:
                    # logger.debug(f'Creating: {m_view_name}')
                    print(f'Refreshing: {self.dbl_qutes_str(mat_view.relname)}')
                    cursor.execute(f'REFRESH MATERIALIZED VIEW {self.dbl_qutes_str(mat_view.relname)};')
                    print(f'Refreshed')
                    # cursor.execute(f'CREATE OR REPLACE FUNCTION tg_refresh_{mat_view.relname}() RETURNS trigger LANGUAGE plpgsql AS $$ BEGIN REFRESH MATERIALIZED VIEW {mat_view.relname}; RETURN NULL; END; $$;')
                    # logger.debug(f'Created')
                    # print(f'Function created')

        except ExceptionOnDocLoading as ex:
            self.root.withdraw()
            logger.error('\n !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n')

            for msg_item in ex.args:
                logger.error(msg_item)

            logger.error('\n !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')



import logging
import os
from os.path import getsize
from shutil import copyfile

from django.conf import settings
from django.core.files import File
from django.db.models import BinaryField, BigIntegerField, CharField, TextField
from django.forms import model_to_dict
from isc_common import delAttr
from isc_common.common import SFTP, FILES_STORE_UNKNOWN_TYPE
from isc_common.common.UploadItemEx import UploadItemEx
from isc_common.fields.files import FileFieldEx, make_REPLACE_FILE_PATH
from isc_common.fields.name_field import NameField
from isc_common.http.DSRequest import DSRequest
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel

logger = logging.getLogger(__name__)


class CryptoQuerySet(CommonManagetWithLookUpFieldsQuerySet):

    def delete(self):
        for item in self:
            Crypto_file.remove_file(item=item)
        return super().delete()


class CryptoManager(CommonManagetWithLookUpFieldsManager):
    def createFromRequest(self, request, function=None):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()
        for key in data:
            if key.find('__') != -1:
                delAttr(_data, key)
        delAttr(_data, 'form')
        if data.get('id') or not data.get('real_name'):
            delAttr(_data, 'id')
            res = super().filter(id=data.get('id')).update(**_data)
            res = model_to_dict(res[0])
            delAttr(res, 'attfile')
            delAttr(res, 'form')
            data.update(res)
        return data

    def updateFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        delAttr(data, 'form')
        super().filter(id=request.get_id()).update(**data)
        return data

    def get_queryset(self):
        return CryptoQuerySet(self.model, using=self._db)


class Crypto_file(AuditModel):
    attfile = FileFieldEx(verbose_name='Файл', max_length=255)
    file_store = CharField(verbose_name='Каталог хранения файла', max_length=255, null=True, blank=True)
    format = NameField(verbose_name='Формат файла')
    key = BinaryField(max_length=200, null=True, blank=True)
    mime_type = NameField(verbose_name='MIME тип файла файла')
    real_name = TextField(verbose_name='Первоначальное имя файла', db_index=True, )
    size = BigIntegerField(verbose_name='Размер файла', default=None)

    object = CryptoManager()

    @property
    def olnly_real_file_name(self):
        ls = self.real_name.split('\\')
        return ls[len(ls) - 1]

    @classmethod
    def exists(cls, filename):
        res = os.path.exists(make_REPLACE_FILE_PATH(filename))
        if res is False:
            logger.debug(f'file: {filename} not exists.')
        return filename, res

    @classmethod
    def check_file(cls, attfile):
        attfile = make_REPLACE_FILE_PATH(str(attfile))
        logger.debug(f'check_file: {attfile}')

        if attfile is None or attfile == '':
            logger.debug(f'file: {attfile} not exists')
            return attfile, False
        else:
            filename = attfile

            if isinstance(settings.FILES_STORE, str):
                _, exists = cls.exists(filename)
                if exists is False or getsize(filename) == 0:
                    logger.debug(f'file: {attfile} not exists')
                    return filename, False

            elif isinstance(settings.FILES_STORE, dict) and settings.FILES_STORE.get('mode') == SFTP:
                ssh_client = settings.SSH_CLIENTS.client(settings.FILES_STORE)
                res = ssh_client.exists(filename)
                if res is False or ssh_client.getsize(filename) == 0:
                    logger.debug(f'file: {attfile} not exists')
                    return filename, False
            else:
                raise Exception(FILES_STORE_UNKNOWN_TYPE)

            logger.debug(f'file: {attfile} exists')
            return filename, True

    @classmethod
    def remove_file(cls, item):
        old_file_store = item.file_store
        file_path = item.attfile.name

        if old_file_store:
            if isinstance(settings.FILES_STORE, str):
                file_path = file_path.replace(old_file_store, settings.FILES_STORE)
            elif isinstance(settings.FILES_STORE, dict) and settings.FILES_STORE.get('mode') == SFTP:
                file_path = file_path.replace(old_file_store, settings.FILES_STORE.get('PATH'))
            else:
                raise Exception(FILES_STORE_UNKNOWN_TYPE)

        if isinstance(settings.REPLACE_FILE_PATH, dict):
            for key, value in settings.REPLACE_FILE_PATH.items():
                file_path = file_path.replace(key, value)

        if file_path:
            if os.altsep:
                file_path = file_path.replace(os.altsep, os.sep)

            if isinstance(settings.FILES_STORE, str):
                os.remove(file_path)
            else:
                settings.SSH_CLIENTS.client(settings.FILES_STORE).remove(settings.FILES_STORE.get('PATH'))
            logger.debug(f'Removed file: {file_path}')

    @classmethod
    def copy_file(cls, uploadItemEx):

        if isinstance(settings.OLD_FILES, dict):
            ssh_client = settings.SSH_CLIENTS.client(settings.OLD_FILES)
            if ssh_client.exists(uploadItemEx.real_file_name):
                ssh_client.get(localpath=uploadItemEx.full_path, remotepath=uploadItemEx.real_file_name)
            else:
                if uploadItemEx.tmp_file_name is not None and os.path.exists(uploadItemEx.tmp_file_name):
                    copyfile(src=uploadItemEx.tmp_file_name, dst=uploadItemEx.full_path)
        else:
            if cls.exists(uploadItemEx.real_file_name):
                copyfile(src=uploadItemEx.real_file_name, dst=uploadItemEx.full_path)
            else:
                if uploadItemEx.tmp_file_name is not None and os.path.exists(uploadItemEx.tmp_file_name):
                    copyfile(src=uploadItemEx.tmp_file_name, dst=uploadItemEx.full_path)

        res = cls.objects.getOptional(real_name=uploadItemEx.real_file_name, size=uploadItemEx.file_size)

        if isinstance(settings.FILES_STORE, dict):
            if res is not None and str(res.attfile) != '' and settings.SSH_CLIENTS.client(settings.FILES_STORE).exists(str(res.attfile)):
                return res, False
        else:
            if res is not None and str(res.attfile) != '' and os.path.exists(str(res.attfile)):
                return res, False

        with open(uploadItemEx.full_path, 'rb') as src:
            fileObj = File(src)

            defaults = dict(
                attfile=fileObj,
                file_store=os.path.split(uploadItemEx.real_file_name)[0],
                format=uploadItemEx.file_format,
                mime_type=uploadItemEx.file_mime_type,
                real_name=uploadItemEx.real_file_name,
                size=uploadItemEx.file_size,
            )

            res, created = cls.objects.update_or_create(
                id=uploadItemEx.id,
                defaults=defaults
            )

        if uploadItemEx.tmp_file_name is not None and os.path.exists(uploadItemEx.tmp_file_name):
            if not os.path.isdir(uploadItemEx.tmp_file_name):
                os.remove(uploadItemEx.tmp_file_name)
            os.remove(uploadItemEx.full_path)

        return res, created

    @classmethod
    def get_action(cls, res, item):
        action = None
        if isinstance(settings.FILES_STORE, dict):
            exists = str(res.attfile) != ''
            if exists is True:
                exists = settings.SSH_CLIENTS.client(settings.FILES_STORE).exists(path=str(res.attfile))
        else:
            exists = os.path.exists(str(res.attfile))

        if res.size is None or res.size == 0 or not exists:
            action = 'create'
        elif res.size != item.file_size:
            action = 'update'
        return action

    @classmethod
    def create_update(cls, **kwargs):
        uploadItemEx = UploadItemEx(**kwargs)

        try:
            if uploadItemEx.id is not None:
                res = cls.objects.get(id=uploadItemEx.id)
                action = cls.get_action(res=res, item=uploadItemEx)
            else:
                if uploadItemEx.real_file_name is None:
                    raise Exception('Must be real_file_name or id')
                else:
                    res = cls.objects.get(real_name=uploadItemEx.real_file_name)

                action = cls.get_action(res=res, item=uploadItemEx)

            if action is not None:
                if action == 'update':
                    cls.remove_file(item=res)

                return cls.copy_file(uploadItemEx=uploadItemEx)
            else:
                return None, None

        except cls.DoesNotExist:
            return cls.copy_file(uploadItemEx=uploadItemEx)
        except Exception as ex:
            raise ex

    def __str__(self):
        return f"{self.real_name}"

    class Meta:
        abstract = True



# Generated by Django 2.2.4 on 2019-08-23 03:57

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0035_delete_calendar_shifts_days'),
    ]

    operations = [
        migrations.AlterField(
            model_name='shift_day',
            name='calendar',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='clndr.Calendars'),
        ),
        migrations.AlterField(
            model_name='shift_day',
            name='numdayinweek',
            field=models.PositiveIntegerField(),
        ),
        migrations.AlterUniqueTogether(
            name='shift_day',
            unique_together={('calendar', 'numdayinweek')},
        ),
    ]



from django.db.models import TextField, NOT_PROVIDED
from django.utils.translation import gettext_lazy as _


class InputMaskField(TextField):
    description = _("Input Mask")

    def get_internal_type(self):
        return "TextField"

    def __init__(self,
                 verbose_name=None,
                 name=None,
                 primary_key=False,
                 max_length=255,
                 unique=False,
                 blank=False,
                 null=False,
                 db_index=False,
                 rel=None,
                 default=NOT_PROVIDED,
                 editable=True,
                 serialize=True,
                 unique_for_date=None,
                 unique_for_month=None,
                 unique_for_year=None,
                 choices=None,
                 help_text='',
                 db_column=None,
                 db_tablespace=None,
                 validators=(),
                 error_messages=None
                 ):
        super().__init__(
            verbose_name=verbose_name,
            name=name,
            primary_key=primary_key,
            max_length=max_length,
            unique=unique,
            blank=blank,
            null=null,
            db_index=db_index,
            rel=rel,
            default=default,
            editable=editable,
            serialize=serialize,
            unique_for_date=unique_for_date,
            unique_for_month=unique_for_month,
            unique_for_year=unique_for_year,
            choices=choices,
            help_text=help_text,
            db_column=db_column,
            db_tablespace=db_tablespace,
            auto_created=False,
            validators=validators,
            error_messages=error_messages
        )



# Generated by Django 3.2.6 on 2021-08-09 14:36

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('react', '0008_page_fragments_pages'),
    ]

    operations = [
        migrations.AddField(
            model_name='pages',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='react.pages'),
        ),
        migrations.AlterField(
            model_name='pages',
            name='code',
            field=isc_common.fields.code_field.CodeStrictField(),
        ),
    ]



import os
import urllib.request

from django.conf import settings
from django.http import HttpResponse

from isc_common.common.UploadItem import UploadItem
from tracker.models.messages_files import Messages_files


def message_download_file(request, id):
    obj = Messages_files.objects.get(id=id)
    if os.path.exists(obj.attfile.name):
        path = obj.attfile.path
    else:
        path = obj.attfile.name
        file_name = path.split(os.sep)[-1]
        path = f'{settings.FILES_STORE}{os.sep}{file_name}'
        if not os.path.exists(path):
            raise Exception(f'Path: {path} not exists.')

    item = UploadItem(stored_file_name=os.path.basename(path), key=obj.key)
    if obj.key:
        path_decrypt = item.decrypt()
    else:
        path_decrypt = item.full_path

    with open(path_decrypt, 'rb') as fh:
        response = HttpResponse(fh.read(), content_type=obj.mime_type)
        # response['Content-Disposition'] = f'attachment; filename="{urllib.request.quote(obj.real_name.encode("utf-8"))}"'
        response['Content-Disposition'] = f'attachment; filename="{urllib.request.quote(obj.real_name)}"'
        response['Content-Length'] = str(os.stat(path_decrypt).st_size)
        fh.close()
        if obj.key:
            os.remove(path_decrypt)
        return response



# Generated by Django 2.2.4 on 2019-08-23 09:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0038_auto_20190823_0948'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='shift_day',
            name='calendar',
        ),
    ]



# Generated by Django 3.1.8 on 2021-04-22 00:01

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0096_auto_20210411_1621'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='images',
            name='sizes',
        ),
        migrations.AddField(
            model_name='image_types',
            name='sizes',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.SmallIntegerField(), blank=True, null=True, size=None),
        ),
    ]



# Generated by Django 3.2.6 on 2021-08-09 15:22

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('react', '0010_fragments_view'),
    ]

    operations = [
        migrations.AddField(
            model_name='page_fragments',
            name='fragment',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='react.fragments'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='page_fragments',
            name='page',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='react.pages'),
            preserve_default=False,
        ),
        migrations.AlterUniqueTogether(
            name='page_fragments',
            unique_together={('page', 'fragment')},
        ),
    ]



# Generated by Django 3.2.7 on 2021-09-12 17:23

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0130_auto_20210907_1904'),
    ]

    operations = [
        migrations.AddField(
            model_name='text_informations',
            name='props',
            field=bitfield.models.BitField((('checked_src_icluded', 'Выполнена проверка на содержащие картинки'),), db_index=True, default=0),
        ),
    ]



from django.urls import path

from isc_common.views import e_mails

urlpatterns = [

    path('E_mails/Fetch/', e_mails.E_mails_Fetch),
    path('E_mails/Add', e_mails.E_mails_Add),
    path('E_mails/Update', e_mails.E_mails_Update),
    path('E_mails/Remove', e_mails.E_mails_Remove),
    path('E_mails/Lookup/', e_mails.E_mails_Lookup),
    path('E_mails/Info/', e_mails.E_mails_Info),
    path('E_mails/Copy', e_mails.E_mails_Copy),

]



import logging

from django.conf import settings
from django.core.management import BaseCommand

from isc_common.auth.models.user import User
from isc_common.ws.webSocket import WebSocket

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def add_arguments(self, parser):
        parser.add_argument('--username', type=str)
        parser.add_argument('--message', type=str)

    def handle(self, *args, **options):
        username = options.get('username')
        user = User.objects.get(username=username)
        message = options.get('message')

        WebSocket.send_info_message(
            host=settings.WS_HOST,
            port=settings.WS_PORT,
            channel=f'common_{user.username}',
            message=message,
            logger=logger
        )



import logging

from django.db.models import Model

logger = logging.getLogger(__name__)


class Model_links(Model):

    @classmethod
    def update_or_create_link(cls, main_model, link_field_name, link, pk_name='id'):
        from isc_common.models.links import Links

        if link != '' or link is not None:
            if eval(f'main_model.{link_field_name} is None', dict(), dict(main_model=main_model)):
                res, _ = eval(f'links.objects.get_or_create(link=link)', dict(), dict(link=link, links=Links))
            else:
                res, _ = eval(f'links.objects.update_or_create(id=main_model.{link_field_name}.id, defaults=dict(link=link))', dict(), dict(link=link, links=Links, main_model=main_model))

            eval(f'main_model._meta.concrete_model.objects.filter({pk_name}=main_model.{pk_name}).update({link_field_name}=res)', dict(), dict(res=res, main_model=main_model))

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Базовый класс'
        abstract = True



import logging

from django.core.management import BaseCommand

from isc_common.auth.models.user import User
from react.models.generators import TypeScriptGenerator

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def handle(self, *args, **options):
        TypeScriptGenerator().generate(data={'pages_ids': [3], 'httpHeaders': {'USER_ID': 7}}, user=User.objects.get(id=7))
        # print(TypeScriptGenerator()._get_text('Фоновый рисунок для всего приложения'))
        print("Done.")



# Generated by Django 3.2.6 on 2021-08-23 12:21

from django.db import migrations


class Migration( migrations.Migration ) :
    dependencies = [
        ('isc_common' , '0109_auto_20210823_1126') ,
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='images' ,
            unique_together={ ('real_name' , 'size') } ,
        ) ,
    ]



from isc_common.http.DSRequest import DSRequest
from isc_common.http.RPCResponse import RPCResponseConstant


class CopyPermission(DSRequest):
    def __init__(self, request):
        from permission.models.usergroup_permission import Usergroup_permission
        from permission.models.user_permission import User_permission

        DSRequest.__init__(self, request)
        data = self.get_data()

        source = data.get('source')
        source_record = source.get('record')
        table_name_source = source.get('table_name')

        desctination = data.get('destination')
        desctination_record = desctination.get('record')
        table_name_destination = desctination.get('table_name')

        if source_record.get('id') == desctination_record.get('id'):
            raise Exception("Источник и приемник не могут совпадать.")

        if source and desctination:
            if table_name_source == 'user_group':
                for user_group_permission in Usergroup_permission.objects.filter(usergroup_id=source_record.get('id')):
                    if table_name_destination == 'user_group':
                        Usergroup_permission.objects.update_or_create(
                            usergroup_id=desctination_record.get('id'),
                            widget=user_group_permission.widget,
                            defaults=dict(permission=user_group_permission.permission)
                        )
                    elif table_name_destination == 'user':
                        User_permission.objects.update_or_create(
                            usergroup_id=desctination_record.get('id'),
                            widget=user_group_permission.widget,
                            defaults=dict(permission=user_group_permission.permission)
                        )
            elif table_name_source == 'user':
                for user_permission in User_permission.objects.filter(usergroup_id=source_record.get('id')):
                    if table_name_destination == 'user_group':
                        Usergroup_permission.objects.update_or_create(
                            usergroup_id=desctination_record.get('id'),
                            widget=user_permission.widget,
                            defaults=dict(permission=user_permission.permission)
                        )
                    elif table_name_destination == 'user':
                        User_permission.objects.update_or_create(
                            usergroup_id=desctination_record.get('id'),
                            widget=user_permission.widget,
                            defaults=dict(permission=user_permission.permission)
                        )

        self.response = dict(status=RPCResponseConstant.statusSuccess)



# Generated by Django 2.2.4 on 2019-08-23 03:50

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0033_auto_20190822_2250'),
    ]

    operations = [
        migrations.AddField(
            model_name='shift_day',
            name='calendar',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='clndr.Calendars'),
        ),
        migrations.AddField(
            model_name='shift_day',
            name='numdayinweek',
            field=models.PositiveIntegerField(default=None),
        ),
    ]



# Generated by Django 3.2.6 on 2021-08-30 14:31

from django.db import migrations, models
import django.db.models.expressions


class Migration(migrations.Migration):

    dependencies = [
        ('react', '0022_auto_20210812_0706'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='fragments',
            constraint=models.CheckConstraint(check=models.Q(('id', django.db.models.expressions.F('parent_id')), _negated=True), name='c_Fragments'),
        ),
    ]



from django.urls import path

from react.views import fragment_param_types

urlpatterns = [

    path('Fragment_param_types/Fetch/', fragment_param_types.Fragment_param_types_Fetch),
    path('Fragment_param_types/Add', fragment_param_types.Fragment_param_types_Add),
    path('Fragment_param_types/Update', fragment_param_types.Fragment_param_types_Update),
    path('Fragment_param_types/Remove', fragment_param_types.Fragment_param_types_Remove),
    path('Fragment_param_types/Lookup/', fragment_param_types.Fragment_param_types_Lookup),
    path('Fragment_param_types/Info/', fragment_param_types.Fragment_param_types_Info),
    path('Fragment_param_types/Copy', fragment_param_types.Fragment_param_types_Copy),

]



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from tracker.models.messages_state import Messages_state, Messages_stateManager


@JsonResponseWithException()
def Messages_state_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Messages_state.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Messages_stateManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_state_Add(request):
    return JsonResponse(DSResponseAdd(data=Messages_state.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_state_Update(request):
    return JsonResponse(DSResponseUpdate(data=Messages_state.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_state_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Messages_state.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)

@JsonResponseWithException()
def Messages_state_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Messages_state.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_state_Info(request):
    return JsonResponse(DSResponse(request=request, data=Messages_state.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)



import logging

from django.core.management import BaseCommand
from django.db import connection

from isc_common.models.pg_class import Pg_class

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    def dbl_qutes_str(self, str):
        return f'"{str}"'

    def handle(self, *args, **options):
        for sequence in Pg_class.objects.filter(relkind='S').order_by('relname'):
            with connection.cursor() as cursor:
                seq_name = sequence.relname
                table_name = seq_name[0:sequence.relname.rfind('_id_seq')]
                try:
                    cursor.execute("select max(id) + 1 from " + table_name)
                    row = cursor.fetchone()
                    id = row[0]

                    if isinstance(id, int):
                        cursor.execute(f'alter sequence {seq_name} restart with {id};')
                        # logger.info(f'seq: {table_name}, id: {id}')
                        print(f'seq: {table_name}, id: {id}')
                except Exception as e:
                    pass
                    # logger.error(e)



from isc_common.auth.models.user import User , UserManager
from isc_common.auth.models.user_view import UserView , UserViewManager
from isc_common.http.DSRequest import DSRequest
from isc_common.http.DSResponse import DSResponseUpdate , DSResponseAdd , DSResponse , JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse


@JsonResponseWithException( printing=False )
def User_Fetch( request ) :
    _request = DSRequest( request )
    return JsonResponse(
        DSResponse(
            request=request ,
            data=UserView.objects.
                exclude( id=0 ).
                distinct().
                get_range_rows11(
                request=request ,
                function=UserManager.getRecord
            ) ,
            status=RPCResponseConstant.statusSuccess ).response )


@JsonResponseWithException( printing=False )
def User_FetchExclBots( request ) :
    return JsonResponse(
        DSResponse(
            request=request ,
            data=UserView.objects.
                exclude( props=User.props.bot ).
                exclude( id=0 ).
                distinct().
                get_range_rows11(
                request=request ,
                function=UserManager.getRecord
            ) ,
            status=RPCResponseConstant.statusSuccess ).response ) @ JsonResponseWithException( printing=False )


def User_Fetch_User_DS4( request ) :
    return JsonResponse(
        DSResponse(
            request=request ,
            data=UserView.objects.
                exclude( props=User.props.bot ).
                exclude( id=0 ).
                distinct().
                get_range_rows1(
                request=request ,
                function=UserViewManager.getRecord
            ) ,
            status=RPCResponseConstant.statusSuccess ).response )


@JsonResponseWithException()
def User_Add( request ) :
    return JsonResponse( DSResponseAdd( data=User.objects.createFromRequest( request=request ) , status=RPCResponseConstant.statusSuccess ).response )


@JsonResponseWithException( printing=False )
def User_Update( request ) :
    return JsonResponse( DSResponseUpdate( data=User.objects.updateFromRequest( request ) , status=RPCResponseConstant.statusSuccess ).response )


@JsonResponseWithException()
def User_Remove( request ) :
    return JsonResponse( DSResponse( request=request , data=User.objects.deleteFromRequest( request=request ) , status=RPCResponseConstant.statusSuccess ).response )


@JsonResponseWithException()
def User_Lookup( request ) :
    return JsonResponse( DSResponse( request=request , data=User.objects.lookupFromRequest( request=request ) , status=RPCResponseConstant.statusSuccess ).response )


@JsonResponseWithException()
def User_Info( request ) :
    return JsonResponse( DSResponse( request=request , data=User.objects.get_queryset().get_info( request=request ) , status=RPCResponseConstant.statusSuccess ).response )



from django.urls import path

from react.views import fragments_4_page_view

urlpatterns = [

    path('Fragments_4_page_view/Fetch/', fragments_4_page_view.Fragments_4_page_view_Fetch),
    path('Fragments_4_page_view/Add', fragments_4_page_view.Fragments_4_page_view_Add),
    path('Fragments_4_page_view/Update', fragments_4_page_view.Fragments_4_page_view_Update),
    path('Fragments_4_page_view/Remove', fragments_4_page_view.Fragments_4_page_view_Remove),
    path('Fragments_4_page_view/Lookup/', fragments_4_page_view.Fragments_4_page_view_Lookup),
    path('Fragments_4_page_view/Info/', fragments_4_page_view.Fragments_4_page_view_Info),
    path('Fragments_4_page_view/Copy', fragments_4_page_view.Fragments_4_page_view_Copy),

]



# Generated by Django 3.0.7 on 2020-06-12 16:23

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('reports', '0006_remove_jasper_reports_users_editor_identifier'),
    ]

    operations = [
        migrations.AddField(
            model_name='jasper_reports_users',
            name='editor_identifier',
            field=isc_common.fields.code_field.CodeField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.1.7 on 2021-03-16 08:27

import django.contrib.auth.validators
from django.db import migrations

import isc_common.fields.code_field


class Migration(migrations.Migration):
    dependencies = [
        ('isc_common', '0086_auto_20210315_1154'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='username',
            field=isc_common.fields.code_field.CodeStrictField(error_messages={'unique': 'Такой пользователь уже существует.'}, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='логин'),
        ),
    ]



from django.urls import path

from react.views import fragments_view

urlpatterns = [

    path('Fragments_view/Fetch/', fragments_view.Fragments_view_Fetch),
    path('Fragments_view/Add', fragments_view.Fragments_view_Add),
    path('Fragments_view/Update', fragments_view.Fragments_view_Update),
    path('Fragments_view/Remove', fragments_view.Fragments_view_Remove),
    path('Fragments_view/Lookup/', fragments_view.Fragments_view_Lookup),
    path('Fragments_view/Info/', fragments_view.Fragments_view_Info),
    path('Fragments_view/Copy', fragments_view.Fragments_view_Copy),

]



import logging

from django.core.management import BaseCommand
from isc_common.models.model_links import Model_links
from isc_common.models.text_informations import Text_informationsManager

logger = logging.getLogger( __name__ )


class Command( BaseCommand ) :
    def handle( self , *args , **options ) :
        print( Model_links.get_link(
            model_id=14451 ,
            model_code='bg_link' ,
            model='competitions_leagues' ,
            model_link='competitions_leagues_links' ,
            model_link_fk='league_id'
        ) )

        print(
            Text_informationsManager.get_text(
                model_id=14394 ,
                model_code='contacts' ,
                model='competitions_leagues' ,
                model_text='competitions_leagues_text_informations' ,
                model_text_fk='league_id'
            )
        )



# Generated by Django 3.2.4 on 2021-06-25 22:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('tracker', '0090_alter_messages_files_attfile'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages_files',
            name='size',
            field=models.BigIntegerField(default=0, verbose_name='Размер файла'),
        ),
    ]



import configparser
import logging
import os
import shutil
import zipfile
from os.path import isdir, isfile, getsize
from tkinter.filedialog import askdirectory

from django.conf import settings
from django.core.management import BaseCommand
from files_blk.models.files import Files
from files_blk.models.files_group import Files_group
from isc_common import replace_alt_set, delete_drive_leter
from isc_common.logger.Logger import Logger
from isc_common.oss.functions import mekeDirs, copyTwo

from ivc.kompas.interaction.interaction_kompas import IsNotDirectory

logger = logging.getLogger(__name__)
logger.__class__ = Logger


class Command(BaseCommand):
    help = "Test"

    config = configparser.ConfigParser()
    config['DEFAULT'] = {}

    config.sections()
    config.read('config.ini')

    def handle(self, *args, **options):
        self.cnt = 0
        self.not_recorded = []

        def rec_file(file):

            dist = f'{settings.EXP_FILES_STORE}{os.sep}{replace_alt_set(delete_drive_leter(file))}'
            file_size = getsize(file)

            try:
                Files.objects.using('files_blk').get(
                    group=self.file_group,
                    path=delete_drive_leter(file),
                    file_size=file_size
                )

                logger.info(f'file exist : {dist}')
            except Files.DoesNotExist:
                dir, _file = os.path.split(dist)
                if mekeDirs(dir, logger, exist_ok=True):
                    if copyTwo(file, dist):
                        self.cnt += 1
                        logger.info(f'file recorded ({self.cnt}) : {dist}')
                    else:
                        self.not_recorded.append(file)
                        logger.warning(f'file not recorded ({self.cnt}) : {dist}')

        def walk_dir(dir):
            if isdir(dir):
                # logger.debug(f'dir: {dir}')
                for path_section in os.listdir(dir):
                    # logger.debug(path_section)
                    full_path = replace_alt_set(f"{dir}{os.sep}{path_section}")
                    if isdir(full_path):
                        walk_dir(dir=full_path)
                    elif isfile(full_path) and full_path.lower().endswith('.spw'):
                        rec_file(file=full_path)
                    elif isfile(full_path) and full_path.lower().endswith('.cdw'):
                        rec_file(file=full_path)
                    elif isfile(full_path) and full_path.lower().endswith('.pdf'):
                        rec_file(file=full_path)
                    elif isfile(full_path) and full_path.lower().endswith('.xml'):
                        rec_file(file=full_path)
            else:
                raise IsNotDirectory(f"{dir} is not directory.")

        def zipdir(path, ziph):
            # ziph is zipfile handle
            for root, dirs, files in os.walk(path):
                for file in files:
                    ziph.write(os.path.join(root, file))

        try:
            try:
                initialdir = self.config['DEFAULT']['file_blk']
                if not os.path.exists(initialdir):
                    initialdir = None
            except KeyError:
                initialdir = None

            self.directory = askdirectory(title='Выбирите каталог размещения файлов', initialdir=initialdir)
            logger.info(f'Выбран каталог : {self.directory}')

            self.full_start_path = f"{settings.EXP_FILES_STORE}{os.sep}{replace_alt_set(delete_drive_leter(self.directory))}"
            if os.path.exists(self.full_start_path):
                shutil.rmtree(self.full_start_path)

            self.file_group, created = Files_group.objects.using('files_blk').get_or_create(code=f'{os.altsep}{delete_drive_leter(self.directory)}')

            walk_dir(dir=replace_alt_set(self.directory))

            zipfname = f'{self.full_start_path}.zip'
            logger.info(f'Start Ziping. ({zipfname})')
            zipf = zipfile.ZipFile(zipfname, 'w')
            zipdir(self.full_start_path, zipf)
            zipf.close()
            logger.info(f'Start Ziped. ({zipfname})')

            self.config['DEFAULT']['file_blk'] = self.directory

            with open('config.ini', 'w') as configfile:
                self.config.write(configfile)

            logger.info(f'Done. ({self.cnt})\n')
            logger.info(f'Not recorded\n')
            for nr in self.not_recorded:
                logger.info(nr)

        except Exception as ex:
            raise ex



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from permission.models.user_permission import User_permission


@JsonResponseWithException()
def User_permission_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=User_permission.objects.
                filter().
                get_range_rows1(
                request=request,
                # function=XXX.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_permission_Add(request):
    return JsonResponse(DSResponseAdd(data=User_permission.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_permission_Update(request):
    return JsonResponse(DSResponseUpdate(data=User_permission.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_permission_Remove(request):
    return JsonResponse(DSResponse(request=request, data=User_permission.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def User_permission_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=User_permission.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



from isc_common.http.DSResponse import DSResponseUpdate , DSResponseAdd , DSResponse , JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from permission.models.widgets_trees import Widgets_trees , Widgets_treesManager


@JsonResponseWithException( printing=False )
def Widgets_trees_Fetch( request ) :
    return JsonResponse(
        DSResponse(
            request=request ,
            data=Widgets_trees.objects.
                filter().
                get_range_rows1(
                request=request ,
                function=Widgets_treesManager.getRecord
            ) ,
            status=RPCResponseConstant.statusSuccess ).response )


@JsonResponseWithException()
def Widgets_trees_Add( request ) :
    return JsonResponse( DSResponseAdd( data=Widgets_trees.objects.createFromRequest( request=request ) , status=RPCResponseConstant.statusSuccess ).response )


@JsonResponseWithException()
def Widgets_trees_Update( request ) :
    return JsonResponse( DSResponseUpdate( data=Widgets_trees.objects.updateFromRequest( request ) , status=RPCResponseConstant.statusSuccess ).response )


@JsonResponseWithException( printing=False )
def Widgets_trees_Remove( request ) :
    return JsonResponse( DSResponse( request=request , data=Widgets_trees.objects.deleteFromRequest( request=request ) , status=RPCResponseConstant.statusSuccess ).response )


@JsonResponseWithException()
def Widgets_trees_Lookup( request ) :
    return JsonResponse( DSResponse( request=request , data=Widgets_trees.objects.lookupFromRequest( request=request ) , status=RPCResponseConstant.statusSuccess ).response )


@JsonResponseWithException()
def Widgets_trees_Info( request ) :
    return JsonResponse( DSResponse( request=request , data=Widgets_trees.objects.get_queryset().get_info( request=request ) , status=RPCResponseConstant.statusSuccess ).response )



from django.urls import path

from isc_common.auth.views import user
from isc_common.auth.views.user import User_Add, User_Lookup, User_Info, User_Remove, User_Update
from isc_common.auth.views.user_download_photo import user_download_photo

urlpatterns = [

    path('User/Fetch/', user.User_Fetch),
    path('User/FetchExclBots/', user.User_FetchExclBots),
    path('logic/User/Fetch_User_DS4/', user.User_Fetch_User_DS4),
    path('User/Add', User_Add),
    path('User/Update', User_Update),
    path('User/Remove', User_Remove),
    path('User/Lookup/', User_Lookup),
    path('User/Info/', User_Info),
    path('User/DownloadPhoto/', user_download_photo),
]



import logging
import time
import uuid

from django.conf import settings
from django.utils import timezone

from isc_common import setAttr
from isc_common.auth.models.user import User
from isc_common.bit import IsBitOn, TurnBitOn, TurnBitOff
from isc_common.datetime import DateToStr
from isc_common.http.DSRequest import DSRequest
from isc_common.ws.webSocket import WebSocket
from tracker.models.messages_state import Messages_state
from tracker.models.messages_theme import Messages_theme
from twits.models.chat_user_user import Chat_user_user
from twits.models.chat_user_user_theme import Chat_user_user_theme
from twits.models.chat_users_view import Chat_usersView

logger = logging.getLogger(__name__)


class LoginRequest(DSRequest):

    @classmethod
    def get_chats(cls, user):
        tetatet_chats_reciver = []
        tetatet_chats_sender = []
        tetatet_chats = []

        key = 'LoginRequest.get_chats'
        settings.LOCKS.acquire(key)
        for chat in Chat_user_user.objects.filter(user_reciver=user):
            tetatet_chats_reciver.append(dict(
                id=chat.common_id,
                props=chat.props._value,
                sender_id=chat.user_sender.id,
                fullname=chat.user_sender.get_full_name,
                shortname=chat.user_sender.get_short_name,
                theme_ids=[item.theme.id for item in Chat_user_user_theme.objects.filter(chat_user_user__common_id=chat.common_id)],
                view_theme_ids=[item.theme.id for item in Chat_user_user_theme.objects.filter(chat_user_user__common_id=chat.common_id) if item.chat_user_user.user_sender == user],
                readonly=True
            ))

        for chat in Chat_user_user.objects.filter(user_sender=user):
            tetatet_chats_sender.append(dict(
                id=chat.common_id,
                props=chat.props._value,
                reciver_id=chat.user_reciver.id,
                fullname=chat.user_reciver.get_full_name,
                shortname=chat.user_reciver.get_short_name,
                theme_ids=[item.theme.id for item in Chat_user_user_theme.objects.filter(chat_user_user__common_id=chat.common_id)],
                view_theme_ids=[item.theme.id for item in Chat_user_user_theme.objects.filter(chat_user_user__common_id=chat.common_id) if item.chat_user_user.user_sender == user],
                readonly=False
            ))

        for chat in tetatet_chats_reciver:
            chat2 = [item for item in tetatet_chats_sender if item.get('id') == chat.get('id')]
            read_only = len(chat2) == 0
            if read_only:
                tetatet_chats.append(chat)
            else:
                compulsory_reading = IsBitOn(chat.get('props'), 0)
                if compulsory_reading:
                    props = TurnBitOn(chat2[0].get('props'), 0)
                else:
                    props = TurnBitOff(chat2[0].get('props'), 0)
                setAttr(chat2[0], 'props', props)
                tetatet_chats.append(chat2[0])

        for chat in tetatet_chats_sender:
            chat2 = [item for item in tetatet_chats if item.get('id') == chat.get('id')]
            if len(chat2) == 0:
                tetatet_chats.append(chat)

        group_chats = [
            dict(
                id=chat_users.chat.id,
                props=chat_users.props._value,
                code=chat_users.chat.code,
                name=chat_users.chat.name,
                readonly=chat_users.readonly,
                theme_ids=[
                    Messages_theme.objects.get_or_create(
                        code=f'group_{chat_users.chat.code}',
                        defaults=dict(
                            name=f'Чат: {chat_users.chat.name}',
                            creator=User.admin_user(),
                            editing=False,
                            deliting=False,
                            parent=Messages_theme.group_chats_theme(),
                            description='Группа создана автоматически'))[0].id
                ],
            ) for chat_users in Chat_usersView.objects.filter(user=user)]
        settings.LOCKS.release(key)

        return dict(
            group_chats=group_chats,
            tetatet_chats=tetatet_chats,
        )

    @classmethod
    def send_bot_message(cls, user, bot, message, compulsory_reading=False):
        _user = user
        _bot = bot

        key = 'LoginRequest.send_bot_message'
        settings.LOCKS.acquire(key)
        if not isinstance(_user, User) and isinstance(_user, int):
            _user = User.objects.get(id=_user)

        if not isinstance(_bot, User) and isinstance(_bot, int):
            _bot = User.objects.get(id=_bot)

        props = TurnBitOn(0, 0) if compulsory_reading else TurnBitOff(0, 0)
        chat_user_user, created = Chat_user_user.objects.get_or_create(
            user_sender=_bot,
            user_reciver=_user,
            defaults=dict(props=props)
        )

        def send():
            channel = f'tetatet_{chat_user_user.common_id}'

            _message = dict(
                channel=channel,
                date=DateToStr(timezone.now(), '%d.%m.%Y, %H:%M:%S', 3),
                guid=uuid.uuid4(),
                host=settings.WS_HOST,
                message=f'<pre>{message}</pre>',
                message_state_delivered_id=Messages_state.message_state_delivered().id,
                message_state_delivered_name=Messages_state.message_state_delivered().name,
                message_state_new_id=Messages_state.message_state_new().id,
                message_state_new_name=Messages_state.message_state_new().name,
                message_state_readed_id=Messages_state.message_state_readed().id,
                message_state_readed_name=Messages_state.message_state_readed().name,
                message_state_not_readed_id=Messages_state.message_state_not_readed().id,
                message_state_not_readed_name=Messages_state.message_state_not_readed().name,
                path='twit',
                port=settings.WS_PORT,
                props=0,
                theme_ids=[item.theme.id for item in Chat_user_user_theme.objects.filter(chat_user_user=chat_user_user)],
                type='message',
                user__color=_bot.color if _bot.color else 'black',
                user__full_name=_bot.get_full_name,
                user__short_name=_bot.get_short_name,
                user_id=_bot.id
            )

            WebSocket.send_message(
                host=settings.WS_HOST,
                port=settings.WS_PORT,
                channel=channel,
                message=_message,
                logger=logger
            )

        if created:
            from twits.models.chats import ChatsManager
            ChatsManager.refresh_chat_menu(user=_user, logger=logger)
            time.sleep(3)
            send()
        else:
            if chat_user_user.props._value != props:
                chat_user_user.props = props
                chat_user_user.save()

                from twits.models.chats import ChatsManager
                ChatsManager.refresh_chat_menu(user=_user, logger=logger)
                time.sleep(3)
            send()
        settings.LOCKS.release(key)



# Generated by Django 2.2.4 on 2019-08-24 12:50

from django.db import migrations
import isc_common.fields.description_field


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0044_auto_20190824_1230'),
    ]

    operations = [
        migrations.AddField(
            model_name='shift_day',
            name='description',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
    ]



# Generated by Django 3.2.6 on 2021-08-23 18:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0116_image_types_key'),
    ]

    operations = [
        migrations.RenameField(
            model_name='image_types',
            old_name='key',
            new_name='keyImage',
        ),
    ]



import logging

from django.contrib.postgres.fields import JSONField
from django.db.models import Model, DateTimeField
from django.utils import timezone

from isc_common.auth.models.user import User
from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyProtect

logger = logging.getLogger(__name__)


class History(Model):
    date = DateTimeField(db_index=True, default=timezone.now)
    method = CodeField()
    path = CodeField()
    user = ForeignKeyProtect(User)
    data = JSONField(default=dict)

    def __str__(self):
        return f"method: {self.method} path: {self.path} data{self.data}"

    class Meta:
        verbose_name = 'History'



# Generated by Django 2.2.3 on 2019-07-28 19:36

from django.db import migrations, models
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0027_delete_history'),
    ]

    operations = [
        migrations.CreateModel(
            name='Mat_view',
            fields=[
                ('schemaname', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('relname', isc_common.fields.code_field.CodeField(blank=True, null=True, primary_key=True, serialize=False)),
                ('ownername', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('refresh_order', models.PositiveIntegerField()),
            ],
            options={
                'verbose_name': 'Mat Views',
                'db_table': 'mat_view_refresh_order',
                'managed': False,
            },
        ),
    ]



import logging
import traceback
from datetime import timedelta

from django.db import transaction
from django.db.utils import DatabaseError, IntegrityError
from django.utils import timezone
from django.utils.deprecation import MiddlewareMixin

from history import utils
from history.models.visitor import Visitor
from history.utils import get_ip

logger = logging.getLogger(__name__)


class VisitorTrackingMiddleware(MiddlewareMixin):
    """
    Keeps track of your active users.  Anytime a visitor accesses a valid URL,
    their unique record will be updated with the page they're on and the last
    time they requested a page.

    Records are considered to be unique when the session key and IP address
    are unique together.  Sometimes the same user used to have two different
    records, so I added a check to see if the session key had changed for the
    same IP and user agent in the last 5 minutes
    """

    def process_request(self, request):
        # create some useful variables
        ip_address = get_ip(request)
        user_agent = request.META.get('HTTP_USER_AGENT')

        if not request.session.session_key:
            request.session.save()
        session_key = request.session.session_key
        username = request.session.get('username')

        # if we get here, the URL needs to be tracked
        # determine what time it is
        now = timezone.now()

        attrs = {
            'username': username,
            'ip_address': ip_address,
        }

        # for some reason, Visitor.objects.get_or_create was not working here
        try:
            visitor = Visitor.objects.using('history').get(**attrs)
        except Visitor.DoesNotExist:
            # see if there's a visitor with the same IP and user agent
            # within the last 5 minutes
            cutoff = now - timedelta(minutes=5)
            visitors = Visitor.objects.using('history').filter(
                ip_address=ip_address,
                user_agent=user_agent,
                last_update__gte=cutoff
            )

            if len(visitors):
                visitor = visitors[0]
                visitor.session_key = session_key
                logger.debug('Using existing visitor for IP %s / UA %s: %s' % (ip_address, user_agent, visitor.id))
            else:
                visitor, created = Visitor.objects.using('history').get_or_create(**attrs)
                if created:
                    logger.debug('Created a new visitor: %s' % attrs)
        except:
            return

        # update the tracking information
        visitor.session_key = session_key
        visitor.user_agent = user_agent

        # if the visitor record is new, or the visitor hasn't been here for
        # at least an hour, update their referrer URL
        one_hour_ago = now - timedelta(hours=1)
        if not visitor.last_update or visitor.last_update <= one_hour_ago:
            visitor.referrer = request.META.get('HTTP_REFERER', 'unknown')
            visitor.session_start = now

        visitor.url = request.path
        visitor.last_update = now
        try:
            sid = transaction.savepoint()
            visitor.save()
            transaction.savepoint_commit(sid)
        except IntegrityError:
            transaction.savepoint_rollback(sid)
        except DatabaseError:
            logger.error('There was a problem saving visitor information:\n%s\n\n%s' % (traceback.format_exc(), locals()))


class VisitorCleanUpMiddleware(MiddlewareMixin):
    """Clean up old visitor tracking records in the database"""

    def process_request(self, request):
        timeout = utils.get_cleanup_timeout()

        if str(timeout).isdigit():
            logger.debug('Cleaning up visitors older than %s hours' % timeout)
            timeout = timezone.localtime(timezone.now()) - timedelta(hours=int(timeout))
            Visitor.objects.using('history').filter(last_update__lte=timeout).delete()



import os

from django.conf import settings
from django.db.models import FileField
from django.db.models.fields.files import FieldFile
from isc_common.fields import Field


def FILE_ex(image, ssh_files):
    attfile = str(image.attfile)
    if attfile != '':
        _, _attfile = os.path.split(attfile)
        attfile = f'{settings.FILES_STORE.get("PATH")}{os.sep}{_attfile}'
        exist = ssh_files.exists(attfile)
        if exist is True:
            image.attfile = _attfile
            image.save()
        return exist, ssh_files.getsize(attfile)
    else:
        return False, 0


def make_REPLACE_FILE_PATH(name):
    if name == '':
        return name

    name = name.replace('\\', os.sep)
    replase_dir = None
    if isinstance(settings.REPLACE_FILE_PATH, dict):
        for key, value in settings.REPLACE_FILE_PATH.items():
            if replase_dir is not None and replase_dir != value:
                raise Exception('Несовпадение параметров замены')
            replase_dir = value
            name = name.replace(key, value)
    _, file_name = os.path.split(name)
    if replase_dir is not None:
        if replase_dir[-1:] != os.sep:
            replase_dir = replase_dir + os.sep
        name = f'{replase_dir}FILES{os.sep}{file_name}'
    return name


def make_REPLACE_IMAGE_PATH(full_name_image):
    full_name_image_replaced = full_name_image.replace(settings.PATH_IMAGES_REPLACE.get('old_path'), settings.PATH_IMAGES_REPLACE.get('new_path'))
    full_name_image_replaced = full_name_image_replaced.replace(os.altsep if os.altsep is not None else '\\', os.sep)
    return full_name_image_replaced


class FieldFileEx(FieldFile, Field):
    def get_replaced_name(self):
        self.name = make_REPLACE_FILE_PATH(name=self.name)
        return self.name

    def open(self, mode='rb'):
        self._require_file()
        if isinstance(settings.REPLACE_FILE_PATH, dict):
            for key, value in settings.REPLACE_FILE_PATH.items():
                self.name = self.name.replace(key, value)
        self.file = self.storage.open(self.name, mode)
        return self

    def save(self, name, content, save=True):
        name = os.path.abspath(name)
        _, file_name = os.path.split(self.storage.save(name, content, max_length=self.field.max_length))
        self.name = file_name
        setattr(self.instance, self.field.name, self.name)

        self._committed = True

        # Save the object because it has changed, unless save is False
        if save:
            self.instance.save()


class FileFieldEx(FileField):
    attr_class = FieldFileEx



# Generated by Django 2.2.4 on 2019-08-05 22:38

from django.conf import settings
from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0031_chat_users_chats_message_user_2_user'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages',
            name='state',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='tracker.Messages_state'),
        ),
        migrations.AlterField(
            model_name='messages',
            name='theme',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='tracker.Messages_theme'),
        ),
        migrations.AlterField(
            model_name='messages',
            name='user',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, related_name='user', to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-22 22:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0015_delete_uuid_2_id'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='files_1c',
            name='id_old',
        ),
    ]



# Generated by Django 3.2.7 on 2021-09-05 12:46

from django.db import migrations
import isc_common.fields.code_field


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0127_auto_20210901_0655'),
    ]

    operations = [
        migrations.AddField(
            model_name='images',
            name='file_name',
            field=isc_common.fields.code_field.CodeField(blank=True, null=True),
        ),
    ]



from django.urls import path

from react.views import fragments

urlpatterns = [

    path('Fragments/Fetch/', fragments.Fragments_Fetch),
    path('Fragments/Add', fragments.Fragments_Add),
    path('Fragments/Add_4_Page', fragments.Fragments_Add_4_Page),
    path('Fragments/Update', fragments.Fragments_Update),
    path('Fragments/Remove', fragments.Fragments_Remove),
    path('Fragments/Lookup/', fragments.Fragments_Lookup),
    path('Fragments/Info/', fragments.Fragments_Info),
    path('Fragments/Copy', fragments.Fragments_Copy),

]



from django.urls import path

from isc_common.views import users_images

urlpatterns = [

    path('Users_images/Fetch/', users_images.Users_images_Fetch),
    path('Users_images/Add', users_images.Users_images_Add),
    path('Users_images/Update', users_images.Users_images_Update),
    path('Users_images/Remove', users_images.Users_images_Remove),
    path('Users_images/Lookup/', users_images.Users_images_Lookup),
    path('Users_images/Info/', users_images.Users_images_Info),
    path('Users_images/Copy', users_images.Users_images_Copy),

]



# Generated by Django 3.2.6 on 2021-08-12 07:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('react', '0021_auto_20210811_1736'),
    ]

    operations = [
        migrations.AddField(
            model_name='fragment_params',
            name='num',
            field=models.SmallIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='fragment_params',
            name='path_file',
            field=models.TextField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.2.6 on 2021-08-23 12:53

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0113_remove_images_image_type'),
    ]

    operations = [

    ]



# Generated by Django 3.2.4 on 2021-07-26 20:06

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('react', '0004_auto_20210726_0736'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='fragments_item_types',
            options={'verbose_name': 'Типы элементов фрагментов'},
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-06 06:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0035_auto_20190806_0644'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Chat_users',
        ),
        migrations.DeleteModel(
            name='Chats',
        ),
    ]



# Generated by Django 3.2.6 on 2021-08-09 15:14

from django.db import migrations, models
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('react', '0009_auto_20210809_1436'),
    ]

    operations = [
        migrations.CreateModel(
            name='Fragments_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeStrictField()),
                ('name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
            ],
            options={
                'verbose_name': 'Структура фрагмента',
                'db_table': 'react_fragments_view',
                'managed': False,
            },
        ),
    ]



import logging

from bitfield import BitField
from django.db.models import BooleanField
from django.forms import model_to_dict

from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditQuerySet, AuditManager, AuditModel
from isc_common.number import DelProps
from twits.models.chat_users import Chat_usersManager
from twits.models.chats import Chats

logger = logging.getLogger(__name__)


class Chat_usersViewQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Chat_usersViewManager(AuditManager):

    @classmethod
    def getRecord(cls, record):
        res = {
            'id': record.id,
            "user_id": record.user.id,
            "chat_id": record.chat.id,
            "user__username": record.user.username,
            "user__first_name": record.user.first_name,
            "user__last_name": record.user.last_name,
            "user__email": record.user.email,
            "user__middle_name": record.user.middle_name,
            'props': record.props,
            'compulsory_reading': record.props.compulsory_reading,
            'relevant': record.props.relevant,
            'readonly': record.props.readonly,
        }
        return DelProps(res)

    def get_queryset(self):
        return Chat_usersViewQuerySet(self.model, using=self._db)

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        user_ids = data.get('user_ids', None)
        context_ids = data.get('context_ids', None)
        _data=[]

        if user_ids and context_ids:
            if not isinstance(user_ids, list):
                user_ids = [user_ids]

            if not isinstance(context_ids, list):
                context_ids = [context_ids]

            for user_id in user_ids:
                for chat_id in context_ids:
                    res = super().create(user_id=user_id, chat_id=chat_id)
                    _data.append(model_to_dict(res))

        return data


class Chat_usersView(AuditModel):
    user = ForeignKeyCascade(User)
    chat = ForeignKeyCascade(Chats)
    props = Chat_usersManager.props()
    compulsory_reading = BooleanField()
    relevant = BooleanField()
    readonly = BooleanField()

    objects = Chat_usersViewManager()

    def __str__(self):
        return f"id: {self.id}, user: [{self.user}] chat: [{self.chat}]"

    class Meta:
        verbose_name = 'Участники чатов'
        unique_together = (('user', 'chat'),)
        managed = False
        db_table = 'twits_chat_users_view'



import logging

from django.utils.deprecation import MiddlewareMixin

logger = logging.getLogger(__name__)


class CorsMiddleware(MiddlewareMixin):
    def process_response(self, request, response):
        setattr(response, "Access-Control-Allow-Origin", "*")
        return response



import logging

from isc_common.auth.models.usergroup import UserGroup
from isc_common.fields.code_field import JSONFieldIVC
from isc_common.fields.related import ForeignKeyCascade
from isc_common.models.audit import AuditModel
from permission.models.widgets_trees import Widgets_trees

logger = logging.getLogger( __name__ )


class Usergroup_permission( AuditModel ) :
    usergroup = ForeignKeyCascade( UserGroup )
    widget = ForeignKeyCascade( Widgets_trees )
    permission = JSONFieldIVC()

    # def __str__(self):
    #     return f"{self.code}: {self.name}"

    class Meta :
        verbose_name = 'Доступы для групп'
        unique_together = (("usergroup" , "widget") ,)



# Generated by Django 2.2.6 on 2019-10-17 10:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0071_messages_theme_admin_view'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='messages_theme_admin_view',
            table='tracker_messages_theme_admin_view',
        ),
    ]



import logging
import os
import re

from django.conf import settings
from django.core.management import BaseCommand
from isc_common.fields.files import make_REPLACE_IMAGE_PATH
from isc_common.models.text_informations import Text_informations
from tqdm import tqdm

logger = logging.getLogger(__name__)


class Command(BaseCommand):

    def handle(self, *args, **options):
        query = Text_informations.objects.filter().order_by('id')
        pbar = tqdm(total=query.count())
        o_ssh_client = settings.SSH_CLIENTS.client(settings.OLD_FILES)

        exp = r'(?i)(src|width|height|alt)=\"(.*?)\"'

        for item in query:
            for name, value in re.findall(exp, item.text):
                if name == 'src' and value.find('http:') == -1:
                    full_name_image_replaced = make_REPLACE_IMAGE_PATH(value)
                    if full_name_image_replaced == value:
                        full_name_image_replaced = f"{settings.PATH_IMAGES_REPLACE.get('new_path')}{value.replace('..', f'{os.sep}lflru')}"

                    if o_ssh_client.exists(full_name_image_replaced):
                        logger.debug(f'\nid: {item.id} {value}')
                    else:
                        logger.debug(f'\n{full_name_image_replaced} not found.')
            pbar.update()



import logging

from django.db.models import BigIntegerField

from isc_common.auth.models.user import User
from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyCascade
from isc_common.models.audit import AuditModel, AuditQuerySet, AuditManager

logger = logging.getLogger(__name__)


class Last_used_processesQuerySet(AuditQuerySet):
    pass


class Last_used_processesManager(AuditManager):
    def get_queryset(self):
        return Last_used_processesQuerySet(self.model, using=self._db)

    @classmethod
    def getRecord(cls, record):
        res = {
            'id': record.id,
        }
        return res


class Last_used_processes(AuditModel):
    user = ForeignKeyCascade(User)
    key = CodeField()
    process = BigIntegerField(db_index=True)

    objects = Last_used_processesManager()

    def __str__(self):
        return f"{self.key}: {self.value}"

    class Meta:
        verbose_name = 'Сохраненные параметры'
        unique_together = (('user', 'key', 'process'),)



import logging

from isc_common import Stack

logger = logging.getLogger( __name__ )


class SSH_Client :
    def __init__( self , hostname , username , password , port=22 ) :
        import paramiko
        import os
        import socket
        import traceback

        hostkey = None
        clients = None
        busy = False

        try :
            host_keys = paramiko.util.load_host_keys(
                os.path.expanduser( "~/.ssh/known_hosts" )
            )
        except IOError :
            try :
                # try ~/ssh/ too, because windows can't have a folder named ~/.ssh/
                host_keys = paramiko.util.load_host_keys(
                    os.path.expanduser( "~/ssh/known_hosts" )
                )
            except IOError :
                logger.debug( "*** Unable to open host keys file" )
                host_keys = { }

        if hostname in host_keys :
            hostkeytype = host_keys[ hostname ].keys()[ 0 ]
            hostkey = host_keys[ hostname ][ hostkeytype ]
            # logger.debug("Using host key of type %s" % hostkeytype)
            # logger.debug("Using host key %s" % str(hostkey))

        try :
            t = paramiko.Transport( (hostname , port) )
            t.connect(
                # hostkey=hostkey,
                username=username ,
                password=password ,
                gss_host=socket.getfqdn( hostname ) ,
                gss_auth=False ,
                gss_kex=False ,
            )
            self.sftp = paramiko.SFTPClient.from_transport( t )
            self.hostname = hostname
            self.username = username
            self.password = password
            self.port = port
            self.busy = False
        except Exception as e :
            logger.error( "*** Caught exception: %s: %s" % (e.__class__ , e) )
            traceback.print_exc()
            raise e

    def __str__(self):
        return f'-p {self.port} {self.username}@{self.hostname}'

    def reconnect( self ) :
        import paramiko
        import socket

        t = paramiko.Transport( (self.hostname , self.port) )
        t.connect(
            # hostkey=hostkey,
            username=self.username ,
            password=self.password ,
            gss_host=socket.getfqdn( self.hostname ) ,
            gss_auth=False ,
            gss_kex=False ,
        )
        self.sftp = paramiko.SFTPClient.from_transport( t )
        self.clients.push(self)
        logger.debug(f'\nReconnect: {str(self)}')

    def get( self , remotepath , localpath , callback=None ) :
        try :
            self.busy = True
            self.sftp.get( remotepath=remotepath , localpath=localpath , callback=callback )
        except IOError as ex :
            logger.error( f'{remotepath} {str( ex )}' )

    def put( self , localpath , remotepath , callback=None , confirm=True ) :
        try :
            self.busy = True
            self.sftp.put( localpath=localpath , remotepath=remotepath , callback=callback , confirm=confirm )
            return None
        except Exception as ex :
            logger.error( ex )
            return ex

    def getcwd( self ) :
        try :
            res = self.sftp.getcwd()
            return res , True
        except Exception as ex :
            logger.error( ex )
            return None , None

    def exists( self , path ) :
        try :
            self.sftp.stat( path )
            return True
        except FileNotFoundError :
            return False
        except OSError:
            self.reconnect()
            self.sftp.stat( path )
            return True


    def getsize( self , path ) :
        if self.exists( path=path ) :
            return self.sftp.stat( path ).st_size
        else :
            None

    def stat( self , path ) :
        if self.exists( path=path ) :
            return self.sftp.stat( path )
        else :
            None

    def open( self , filename , mode="r" , bufsize=-1 ) :
        if self.exists( path=filename ) :
            self.busy = True
            return self.sftp.open( filename=filename , mode=mode , bufsize=bufsize )
        else :
            None

    def remove( self , path ) :
        if self.exists( path=path ) :
            self.sftp.remove( path=path )
        else :
            None

    def chmod( self , path , mode ) :
        if self.exists( path=path ) :
            self.sftp.chmod( path=path , mode=mode )
        else :
            None

    def close( self ) :
        self.sftp.close()


class SSH_Clients :
    max_connect = 10;

    clients = Stack()

    def client( self , client_connect ) :
        # logger.debug(f'request connetc: {client_connect}')
        logger.debug( f'lenght stack: {self.clients.size()}' )

        if self.clients.size() > self.max_connect :
            for connect in self.clients.stack[ : - self.max_connect // 2 ] :
                connect.close()

            self.clients.stack = self.clients.stack[ : -self.max_connect // 2 ]
            logger.debug( f'lenght stack: {self.clients.size()}' )

        # clients = list(filter(lambda x: x.busy is False, self.clients.find(lambda client: client.hostname == client_connect.get('SSH_HOST') and
        #                                                                                   client.port == client_connect.get('SSH_PORT') and
        #                                                                                   client.username == client_connect.get('SSH_USER') and
        #                                                                                   client.password == client_connect.get('SSH_PASSWORD')
        #                                                                    )))
        # if len(clients) > 0:
        #     return clients[0]
        # else:
        res = SSH_Client(
            hostname=client_connect.get( 'SSH_HOST' ) ,
            username=client_connect.get( 'SSH_USER' ) ,
            password=client_connect.get( 'SSH_PASSWORD' ) ,
            port=client_connect.get( 'SSH_PORT' )
        )
        res.clients = self.clients
        self.clients.push( res )
        return res

    def close( self ) :
        for client in self.clients :
            client.close()



import logging

from isc_common.fields.code_field import CodeStrictField
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet

logger = logging.getLogger(__name__)


class Model_e_mailQuerySet(AuditQuerySet):

    def update_or_create(self, defaults=None, **kwargs):
        e_mail = defaults.get('e_mail')

        if e_mail != '' and e_mail is not None:
            e_mail_item = E_mails.objects.getOptional(e_mail=e_mail)
            if e_mail_item is None:
                try:
                    item = super().get(**kwargs)
                    e_mail_item, _ = E_mails.objects.update_or_create(id=item.e_mail.id, defaults=dict(e_mail=e_mail))
                except self.model.DoesNotExist:
                    e_mail_item = E_mails.objects.create(e_mail=e_mail)
            defaults = dict(e_mail=e_mail_item)
            return super().update_or_create(defaults=defaults, **kwargs)
        else:
            return None, None


class E_mailsQuerySet(AuditQuerySet):
    pass


class E_mailsManager(AuditManager):

    @classmethod
    def getRecord(cls, record ) :
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return E_mailsQuerySet(self.model, using=self._db)


class E_mails(AuditModel):
    e_mail = CodeStrictField(unique=True)
    objects = E_mailsManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Почтовые адреса'



from django.db.models import PROTECT, CASCADE, ForeignKey, SET_NULL


class ForeignKeyProtect(ForeignKey):
    def __init__(self,
                 to,
                 on_delete=PROTECT,
                 related_name=None,
                 related_query_name=None,
                 limit_choices_to=None,
                 parent_link=False,
                 to_field=None,
                 db_constraint=True,
                 **kwargs):
        super().__init__(to,
                         on_delete,
                         related_name,
                         related_query_name,
                         limit_choices_to,
                         parent_link,
                         to_field,
                         db_constraint,
                         **kwargs)


class ForeignKeyCascade(ForeignKey):
    def __init__(self,
                 to,
                 on_delete=CASCADE,
                 related_name=None,
                 related_query_name=None,
                 limit_choices_to=None,
                 parent_link=False,
                 to_field=None,
                 db_constraint=True,
                 **kwargs):
        super().__init__(to,
                         on_delete,
                         related_name,
                         related_query_name,
                         limit_choices_to,
                         parent_link,
                         to_field,
                         db_constraint,
                         **kwargs)


class ForeignKeySetNull(ForeignKey):
    def __init__(self,
                 to,
                 on_delete=SET_NULL,
                 related_name=None,
                 related_query_name=None,
                 limit_choices_to=None,
                 parent_link=False,
                 to_field=None,
                 db_constraint=True,
                 **kwargs):
        super().__init__(to,
                         on_delete,
                         related_name,
                         related_query_name,
                         limit_choices_to,
                         parent_link,
                         to_field,
                         db_constraint,
                         **kwargs)



import logging

from bitfield import BitField
from django.db import connection
from django.db.models import TextField

from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet

logger = logging.getLogger(__name__)


class Model_text_informationsQuerySet(AuditQuerySet):

    def update_or_create(self, defaults=None, **kwargs):
        text = defaults.get('text')

        if text != '' and text is not None:
            try:
                item = super().get(**kwargs)
                text_item, _ = Text_informations.objects.update_or_create(id=item.id, defaults=dict(text=text))
            except self.model.DoesNotExist:
                text_informations = list(Text_informations.objects.filter(text=text))
                if len(text_informations) > 0:
                    text_item = text_informations[0]
                else:
                    text_item = Text_informations.objects.create(text=text)

            defaults = dict(text=text_item)
            return super().update_or_create(defaults=defaults, **kwargs)


class Text_informationsQuerySet(AuditQuerySet):
    pass


class Text_informationsManager(AuditManager):
    @classmethod
    def props(cls):
        return BitField(flags=(
            ('checked_src_icluded', 'Выполнена проверка на содержащие картинки'),  # 1
        ), default=0, db_index=True)

    @classmethod
    def get_text( cls , model_id , model_code , model , model_text , model_text_fk ) :
        with connection.cursor() as cursor :
            cursor.execute( f'''select cml.id, cml.text
                                from {model} as lg
                                join {model_text} as lgl on lg.id = lgl.{model_text_fk}
                                join isc_common_text_informations as cml on lgl.text_id = cml.id
                                where lg.id = %s
                                and lgl.code = %s''' , [ model_id , model_code ] )
            row = cursor.fetchone()
            if row is None :
                return None , None
            else :
                id , text = row
            return id , text

    @classmethod
    def getRecord(cls, record ) :
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
            'text': record.text,
        }
        return res

    def get_queryset(self):
        return Text_informationsQuerySet(self.model, using=self._db)


class Text_informations(AuditModel):
    props = Text_informationsManager.props()
    text = TextField(db_index=True)

    objects = Text_informationsManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Якоря'



import logging

from django.db.models import BooleanField

from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.base_ref import BaseRefManager, BaseRefQuerySet, BaseRefHierarcy
from react.models.page_level_types import Page_level_types
from react.models.pages import PagesManager

logger = logging.getLogger(__name__)


class Pages_viewQuerySet(BaseRefQuerySet):
    pass


class Pages_viewManager(BaseRefManager):

    @classmethod
    def getRecord(cls, record ) :
        res = {
            'application': record.application,
            'code': record.code,
            'deliting': record.deliting,
            'description': record.description,
            'editing': record.editing,
            'id': record.id,
            'name': record.name,
            'parent_id': record.parent.id if record.parent else None,
            'props': record.props,
            'type__name': record.type.name,
            'type_id': record.type.id,
        }
        return res

    def get_queryset(self):
        return Pages_viewQuerySet(self.model, using=self._db)


class Pages_view(BaseRefHierarcy):
    application = BooleanField()
    props = PagesManager.props()
    type = ForeignKeyProtect(Page_level_types)

    objects = Pages_viewManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Структура страницы'
        db_table = 'react_pages_view'
        managed = False



from isc_common.common.functions import get_relation_field_name
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException, JsonWSResponseWithException, JsonWSPostResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from tracker.models.messages import Messages, MessagesManager
from tracker.views.message_confirm_reading import DSResponse__Messages_Confirm_reading
from tracker.views.message_download_file import message_download_file
from tracker.views.messages_upload1_file import DSResponse__Messages_UploadFile1
from tracker.views.messages_upload_file import DSResponse__Messages_UploadFile


@JsonResponseWithException(printing=False)
def Messages_Fetch(request):
    response = DSResponse(
        request=request,
        data=Messages.objects.
            select_related(*get_relation_field_name( model=Messages )).
            get_range_rows1(
            request=request,
            function=MessagesManager.getRecord
        ),
        status=RPCResponseConstant.statusSuccess).response
    return JsonResponse(response)


@JsonResponseWithException(printing=False)
def Messages_Fetch4Chat(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Messages.objects.
                get_range_rows1(
                request=request,
                function=MessagesManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_Add(request):
    return JsonResponse(DSResponseAdd(data=Messages.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_Add_Auto_Error(request):
    return JsonResponse(DSResponseAdd(data=Messages.objects.createAutoErrorFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_Update(request):
    return JsonResponse(DSResponseUpdate(data=Messages.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Messages.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Messages_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Messages.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException(printing=False)
def Messages_Info(request):
    return JsonResponse(DSResponse(request=request, data=Messages.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonWSResponseWithException()
def Messages_UploadFile(request):
    return JsonResponse(DSResponse__Messages_UploadFile(request).response)


@JsonWSResponseWithException()
def Messages_UploadFile1(request):
    response = DSResponse__Messages_UploadFile1(request).response
    return JsonResponse(response)


@JsonWSPostResponseWithException()
def Messages_Confirm_reading(request):
    return JsonResponse(DSResponse__Messages_Confirm_reading(request).response)


@JsonWSResponseWithException(printing=False)
def Message_DownloadFile(request, id):
    return message_download_file(request, id)



# Generated by Django 3.1.7 on 2021-03-09 09:19

import bitfield.models
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

import isc_common.fields.related


class Migration(migrations.Migration):
    dependencies = [
        # ('competitions', '0158_auto_20210309_0833'),
        ('isc_common', '0077_image_types_images_links_text_informations'),
    ]

    operations = [
        migrations.CreateModel(
            name='Users_images',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('props', bitfield.models.BitField((('main_photo', 'main_photo'),), db_index=True, default=0)),
                ('club', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='competitions.clubs')),
                ('image', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='isc_common.images')),
                ('main_model', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Кросс таблица',
            },
        ),
        migrations.AddConstraint(
            model_name='users_images',
            constraint=models.UniqueConstraint(condition=models.Q(club=None), fields=('image', 'main_model', 'props'), name='Users_images_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='users_images',
            constraint=models.UniqueConstraint(fields=('club', 'image', 'main_model', 'props'), name='Users_images_unique_constraint_3'),
        ),
    ]



from channels.generic.websocket import AsyncWebsocketConsumer


class Consumer(AsyncWebsocketConsumer):
    async def connect(self):
        kwargs = self.scope['url_route']['kwargs']
        channel = kwargs.get('channel')

        self.group = f'group_{channel}'

        # Join room group
        await self.channel_layer.group_add(self.group, self.channel_name)

        await self.accept()

    async def disconnect(self, close_code):
        # Leave room group
        await self.channel_layer.group_discard(self.group, self.channel_name)

    async def receive(self, text_data=None, bytes_data=None, **kwargs):
        if text_data:
            await self.channel_layer.group_send(
                self.group,
                {
                    'type': 'common',
                    'text_data': text_data
                }
            )
        else:
            raise ValueError("No text section for incoming WebSocket frame!")

    # Receive message from room group
    async def common(self, event):
        text_data = event['text_data']

        # Send message to WebSocket
        await self.send(text_data=text_data)



# Generated by Django 2.2.1 on 2019-05-17 18:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0023_fill_Messages_whom'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='messages',
            name='to_whom',
        ),
    ]



from django.urls import path

from react.views import page_fragments

urlpatterns = [

    path('Page_fragments/Fetch/', page_fragments.Page_fragments_Fetch),
    path('Page_fragments/Add', page_fragments.Page_fragments_Add),
    path('Page_fragments/Update', page_fragments.Page_fragments_Update),
    path('Page_fragments/Remove', page_fragments.Page_fragments_Remove),
    path('Page_fragments/Lookup/', page_fragments.Page_fragments_Lookup),
    path('Page_fragments/Info/', page_fragments.Page_fragments_Info),
    path('Page_fragments/Copy', page_fragments.Page_fragments_Copy),

]



import logging

from bitfield import BitField
from django.db.models import CheckConstraint , F , Q
from isc_common import delAttr
from isc_common.bit import IsBitOn
from isc_common.fields.code_field import CodeStrictField
from isc_common.http.DSRequest import DSRequest
from isc_common.models.base_ref import BaseRefHierarcy , BaseRefManager , BaseRefQuerySet

logger = logging.getLogger( __name__ )


class Event_typesQuerySet( BaseRefQuerySet ) :
    pass


class Event_typesManager( BaseRefManager ) :

    @classmethod
    def getRecord( cls , record ) :
        res = {
            'id' : record.id ,
            'code' : record.code ,
            'name' : record.name ,
            'description' : record.description ,
            'parent_id' : record.parent.id if record.parent else None ,
            'props' : record.props._value ,
            'isEvent' : IsBitOn( record.props._value , 0 ) ,
            'compulsory_reading' : IsBitOn( record.props._value , 1 ) ,
        }
        return res

    def get_queryset( self ) :
        return Event_typesQuerySet( self.model , using=self._db )

    def updateFromRequest( self , request , removed=None , function=None ) :
        request = DSRequest( request=request )
        data = request.get_data()
        _data = data.copy()
        delAttr( _data , 'id' )
        delAttr( _data , 'isEvent' )
        delAttr( _data , 'isFolder' )
        delAttr( _data , 'compulsory_reading' )

        res = super().filter( id=data.get( 'id' ) ).update( **_data )
        return data


class Event_types( BaseRefHierarcy ) :
    code = CodeStrictField( unique=True )
    props = BitField( flags=(
        ('isEvent' , 'Является событием') ,  # 1
        ('compulsory_reading' , 'Обязательное прочтение') ,  # 2
    ) , default=0 , db_index=True )

    objects = Event_typesManager()

    def __str__( self ) :
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}"

    class Meta :
        verbose_name = 'Типы событий'
        constraints = [
            CheckConstraint( check=~Q( id=F( 'parent_id' ) ) , name=f'c_Event_types' ) ,
        ]



# Generated by Django 2.2.2 on 2019-06-15 09:10

from django.db import migrations
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('history', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='history',
            name='username',
            field=isc_common.fields.name_field.NameField(),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-22 22:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0028_mat_view'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='params',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='user',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='user_permission',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='usergroup',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='usergroup_permission',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='widgets_trees',
            name='id_old',
        ),
    ]



import logging

from django.db.models import CheckConstraint , F , Q
from django.db.models import SmallIntegerField , CharField
from isc_common.fields.code_field import CodeField
from isc_common.models.base_ref import BaseRefHierarcy , BaseRefManager , BaseRefQuerySet

logger = logging.getLogger( __name__ )


class Image_typesQuerySet( BaseRefQuerySet ) :
    def create( self , **kwargs ) :
        if kwargs.get( 'keyimage' ) == 'clothes' :
            raise Exception( f'Try write: {kwargs.get( "keyimage" )}' )
        return super().create( **kwargs )


class Image_typesManager( BaseRefManager ) :

    @classmethod
    def getRecord( cls , record ) :
        res = {
            'code' : record.code ,
            'deliting' : record.deliting ,
            'description' : record.description ,
            'editing' : record.editing ,
            'id' : record.id ,
            'name' : record.name ,
            'parent' : record.parent.id if record.parent else None ,
        }
        return res

    def get_queryset( self ) :
        return Image_typesQuerySet( self.model , using=self._db )


class Image_types( BaseRefHierarcy ) :
    code = CodeField( db_index=True )
    keyimage = CharField( max_length=255 , null=True , blank=True )
    height = SmallIntegerField( null=True , blank=True )
    width = SmallIntegerField( null=True , blank=True )

    objects = Image_typesManager()

    def __str__( self ) :
        return f'ID:{self.id} code: {self.code} name: {self.name} height: {self.height} width: {self.width} keyimage: {self.keyimage}'

    def __repr__( self ) :
        return self.__str__()

    class Meta :
        verbose_name = 'Image types'
        unique_together = (('code' , 'keyimage') ,)
        constraints = [
            CheckConstraint( check=~Q( id=F( 'parent_id' ) ) , name=f'c_Event' ) ,
        ]



import logging
import os
from os.path import expanduser

from django.conf import settings
from django.db import transaction
from isc_common import Stack
from isc_common.fields.files import make_REPLACE_FILE_PATH, make_REPLACE_IMAGE_PATH
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from isc_common.number import flen

logger = logging.getLogger(__name__)


class Model_imagesQuerySet(AuditQuerySet):
    pass


class Model_imagesManager(AuditManager):

    @classmethod
    def getRecord(cls, record):
        res = {
            'id': record.id,
        }
        return res

    def get_queryset(self):
        return Model_imagesQuerySet(self.model, using=self._db)


class ImageNotFound(Exception):
    pass


class Model_images(AuditModel):
    @classmethod
    def get_image(cls, file_name, path=None):
        if file_name is None:
            return None

        if os.altsep is not None:
            have_path = file_name.find(os.altsep) != -1
        else:
            have_path = file_name.find(os.sep) != -1

        if have_path:
            if os.altsep is None:
                file_name1 = f'{settings.OLD_SITE_BASE_DIR}{file_name}'.replace(os.sep, '\\')
            else:
                file_name1 = f'{settings.OLD_SITE_BASE_DIR}{file_name}'.replace(os.altsep, os.sep)
            try:
                return cls.objects.get(path__upper=file_name1.upper()).path
            except cls.DoesNotExist:
                return None

        query = cls.objects.filter(file_name=file_name).order_by('-date')
        if query.count() == 0:
            return None
        else:
            _list = list(map(lambda x: x.path, query))
            if path is None:
                return _list[0]

            for item in _list:
                if item.find(path) != -1:
                    return item
            return None

    @classmethod
    def rec_update(cls, main_model, image, code, keyimage, result=None, self_image_field=False, defaults=None):
        if self_image_field is False:
            from isc_common.models.image_types import Image_types
            image_type, _ = Image_types.objects.get_or_create(code=code, keyimage=keyimage)

            if isinstance(main_model, int):
                res, _ = cls.objects.update_or_create(image=image, type=image_type, main_model_id=main_model, defaults=defaults)
            else:
                res, _ = cls.objects.update_or_create(image=image, type=image_type, main_model=main_model, defaults=defaults)

            if result is not None:
                result.push(res, lambda item, stack: flen(filter(lambda x: x.id == res.id, stack)) > 0)
        else:
            raise Exception('Not implementation')
            # eval( f'main_model._meta.concrete_model.objects.filter({pk_name}=main_model.{pk_name}).update({image_field_name}=image)' , dict() , dict( image=image , main_model=main_model ) )

    @classmethod
    def update_or_create_image(cls, main_model, path, file_name, keyimage, self_image_field=False, code=None, exception=True, defaults=None, o_ssh_client=None):
        from isc_common.models.images import Images
        from lfl_admin.common.models.site_lfl_images import Site_lfl_images

        result = Stack()

        if file_name is None or file_name.strip() == '':
            return result

        if (code is None or code.strip() == '') or (path is None or path.strip() == ''):
            code = path

        with transaction.atomic():
            full_name_images = Site_lfl_images.get_image(file_name=file_name, path=path)

            if len(full_name_images) == 0 and file_name is not None:
                if exception is True:
                    raise ImageNotFound(f'{file_name} : not found')
                # else :
                #     logger.debug( f'{file_name} : not found !!!' )

            for full_name_image in full_name_images:
                if full_name_image is not None:

                    zero = False

                    full_name_image_replaced = make_REPLACE_IMAGE_PATH(full_name_image)
                    home = expanduser("~")
                    localpath = f"{home}/tmp"
                    if not os.path.exists(localpath):
                        os.makedirs(localpath)

                    if o_ssh_client is None:
                        o_ssh_client = settings.SSH_CLIENTS.client(settings.OLD_FILES)

                    if o_ssh_client.exists(full_name_image_replaced):
                        size = o_ssh_client.getsize(full_name_image_replaced)
                        if size == 0:
                            image.real_name = full_name_image_replaced
                            image.size = 0
                            image.attfile = ''
                            image.save()
                            continue

                        _, _file_name = os.path.split(full_name_image_replaced)
                        # logger.debug( f'file_name: {_file_name}, size: {size}' )
                        image = Images.objects.getOptional(file_name=_file_name, size=size)
                        if image is None:
                            image = Images.objects.getOptional(real_name=full_name_image_replaced, size=size)

                            if (image is not None):
                                image.file_name = _file_name
                                image.size = size
                                image.save()
                            else:
                                image = Images.objects.create(real_name=full_name_image_replaced, file_name=_file_name, size=size)

                    else:
                        zero = True
                        logger.debug(f'{full_name_image_replaced} : not found')

                    if not zero:
                        cls.rec_update(
                            code=code,
                            defaults=defaults,
                            image=image,
                            keyimage=keyimage,
                            main_model=main_model,
                            result=result,
                            self_image_field=self_image_field,
                        )
                        # logger.debug( f'{file_name} : found' )

            return result

    @classmethod
    def replace_image(cls, image, _image, django_model_images, imports, main_model, keyimage):
        for _import in imports:
            eval(f'exec("{_import}")')

        s4compile = f'''{django_model_images}.objects.filter(image_id={image.id}).update(image_id={_image.id})'''
        code = compile(s4compile, '<string>', 'eval')
        eval(code)
        cls.rec_update(
            code=code,
            image=_image,
            main_model=main_model,
            keyimage=keyimage,
        )

    @classmethod
    def update_or_create_image1(cls, main_model, code, path, keyimage, image_id, django_model_images, imports, defaults=None):
        from isc_common.models.images import Images

        imports = imports.split(',')

        if image_id is None:
            return None

        with transaction.atomic():
            image = Images.objects.get(id=image_id)
            full_name_image = image.real_name
            if full_name_image is not None:

                full_name_image_replaced = make_REPLACE_IMAGE_PATH(full_name_image)

                localpath = f"{expanduser('~')}/tmp"
                if not os.path.exists(localpath):
                    os.makedirs(localpath)

                o_ssh_clients = settings.SSH_CLIENTS.client(settings.OLD_FILES)
                if o_ssh_clients.exists(full_name_image_replaced):
                    size = o_ssh_clients.getsize(full_name_image_replaced)
                    _, fileName = os.path.split(full_name_image_replaced)

                    _image = Images.objects.getOptional(real_name=full_name_image_replaced, size=size)
                    if _image is None:
                        _image = Images.objects.getOptional(real_name=fileName, size=size)
                    else:
                        _image.real_name = fileName;
                        _image.save()

                    if _image is not None:
                        attfile = make_REPLACE_FILE_PATH(str(_image.attfile))

                        if isinstance(settings.FILES_STORE, dict):
                            exists = settings.SSH_CLIENTS.client(settings.FILES_STORE).exists(attfile)
                        else:
                            exists = os.path.exists(attfile)

                        if str(attfile) != '' and exists is True:
                            cls.replace_image(
                                image=image,
                                _image=_image,
                                django_model_images=django_model_images,
                                imports=imports,
                                main_model=main_model,
                                keyimage=keyimage,
                            )
                            return _image.id
                        else:
                            _image.size = 0
                            _image.save()

                    image1, created = Images.create_update(
                        id=image.id,
                        real_file_name=full_name_image_replaced,
                        tmp_dir_name=localpath,
                    )

                    if image1.id != image.id:
                        cls.replace_image(
                            image=image,
                            _image=image1,
                            django_model_images=django_model_images,
                            imports=imports,
                            main_model=main_model,
                            keyimage=keyimage,
                        )
                        return image1.id
                else:
                    return None

                cls.rec_update(
                    code=code,
                    keyimage=keyimage,
                    image=image,
                    main_model=main_model,
                    defaults=defaults,
                )
                return image.id
            return image.id

    objects = Model_imagesManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Базовый класс'
        abstract = True



from django.urls import path

from tracker.views import messages_state

urlpatterns = [

    path('Messages_state/Fetch/', messages_state.Messages_state_Fetch),
    path('Messages_state/Add', messages_state.Messages_state_Add),
    path('Messages_state/Update', messages_state.Messages_state_Update),
    path('Messages_state/Remove', messages_state.Messages_state_Remove),
    path('Messages_state/Lookup/', messages_state.Messages_state_Lookup),
    path('Messages_state/Info/', messages_state.Messages_state_Info),

]
